@@ -7,11 +7,10 @@
 """Write Files: write arbitrary files"""
 
 import base64
+import logging
 import os
-from logging import Logger
 from textwrap import dedent
 
-from cloudinit import log as logging
 from cloudinit import util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
@@ -37,15 +36,18 @@ meta: MetaSchema = {
         before being written. For empty file creation, content can be omitted.
 
     .. note::
-        if multiline data is provided, care should be taken to ensure that it
-        follows yaml formatting standards. to specify binary data, use the yaml
+        If multiline data is provided, care should be taken to ensure that it
+        follows yaml formatting standards. To specify binary data, use the yaml
         option ``!!binary``
 
     .. note::
         Do not write files under /tmp during boot because of a race with
         systemd-tmpfiles-clean that can cause temp files to get cleaned during
         the early boot process. Use /run/somedir instead to avoid race
-        LP:1707222."""
+        LP:1707222.
+
+    .. warning::
+       Existing files will be overridden."""
     ),
     "distros": ["all"],
     "examples": [
@@ -72,7 +74,7 @@ meta: MetaSchema = {
         ),
         dedent(
             """\
-        # Provide gziped binary content
+        # Provide gzipped binary content
         write_files:
         - encoding: gzip
           content: !!binary |
@@ -116,9 +118,7 @@ meta: MetaSchema = {
 __doc__ = get_meta_doc(meta)
 
 
-def handle(
-    name: str, cfg: Config, cloud: Cloud, log: Logger, args: list
-) -> None:
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     file_list = cfg.get("write_files", [])
     filtered_files = [
         f
@@ -126,7 +126,7 @@ def handle(
         if not util.get_cfg_option_bool(f, "defer", DEFAULT_DEFER)
     ]
     if not filtered_files:
-        log.debug(
+        LOG.debug(
             "Skipping module named %s,"
             " no/empty 'write_files' key in configuration",
             name,
@@ -146,7 +146,7 @@ def canonicalize_extraction(encoding_type):
     # Yaml already encodes binary data as base64 if it is given to the
     # yaml file as binary, so those will be automatically decoded for you.
     # But the above b64 is just for people that are more 'comfortable'
-    # specifing it manually (which might be a possibility)
+    # specifying it manually (which might be a possibility)
     if encoding_type in ["b64", "base64"]:
         return ["application/base64"]
     if encoding_type == TEXT_PLAIN_ENC:
@@ -179,7 +179,9 @@ def write_files(name, files, owner: str):
         (u, g) = util.extract_usergroup(f_info.get("owner", owner))
         perms = decode_perms(f_info.get("permissions"), DEFAULT_PERMS)
         omode = "ab" if util.get_cfg_option_bool(f_info, "append") else "wb"
-        util.write_file(path, contents, omode=omode, mode=perms)
+        util.write_file(
+            path, contents, omode=omode, mode=perms, user=u, group=g
+        )
         util.chownbyname(path, u, g)
 
 
@@ -214,6 +216,3 @@ def extract_contents(contents, extraction_types):
         elif t == TEXT_PLAIN_ENC:
             pass
     return result
-
-
-# vi: ts=4 expandtab
