@@ -2,11 +2,11 @@
 
 import copy
 import glob
+import logging
 import os
 import re
 from typing import Optional
 
-from cloudinit import log as logging
 from cloudinit import subp, util
 from cloudinit.net import ParserError, renderer, subnet_is_ipv6
 from cloudinit.net.network_state import NetworkState
@@ -81,7 +81,7 @@ def _iface_add_subnet(iface, subnet):
         if key == "address":
             value = "%s/%s" % (subnet["address"], subnet["prefix"])
         if value and key in valid_map:
-            if type(value) == list:
+            if isinstance(value, list):
                 value = " ".join(value)
             if "_" in key:
                 key = key.replace("_", "-")
@@ -126,7 +126,7 @@ def _iface_add_attrs(iface, index, ipv4_subnet_mtu):
 
     for key, value in iface.items():
         # convert bool to string for eni
-        if type(value) == bool:
+        if isinstance(value, bool):
             value = "on" if iface[key] else "off"
         if not value or key in ignore_map:
             continue
@@ -144,7 +144,7 @@ def _iface_add_attrs(iface, index, ipv4_subnet_mtu):
             for v in value:
                 content.append("    {0} {1}".format(renames.get(key, key), v))
             continue
-        if type(value) == list:
+        if isinstance(value, list):
             value = " ".join(value)
         content.append("    {0} {1}".format(renames.get(key, key), value))
 
@@ -179,7 +179,7 @@ def _parse_deb_config_data(ifaces, contents, src_dir, src_path):
     """Parses the file contents, placing result into ifaces.
 
     '_source_path' is added to every dictionary entry to define which file
-    the configration information came from.
+    the configuration information came from.
 
     :param ifaces: interface dictionary
     :param contents: contents of interfaces file
@@ -308,18 +308,6 @@ def _parse_deb_config_data(ifaces, contents, src_dir, src_path):
             ifaces[iface]["auto"] = False
 
 
-def parse_deb_config(path):
-    """Parses a debian network configuration file."""
-    ifaces = {}
-    with open(path, "r") as fp:
-        contents = fp.read().strip()
-    abs_path = os.path.abspath(path)
-    _parse_deb_config_data(
-        ifaces, contents, os.path.dirname(abs_path), abs_path
-    )
-    return ifaces
-
-
 def convert_eni_data(eni_data):
     # return a network config representation of what is in eni_data
     ifaces = {}
@@ -329,7 +317,7 @@ def convert_eni_data(eni_data):
 
 def _ifaces_to_net_config_data(ifaces):
     """Return network config that represents the ifaces data provided.
-    ifaces = parse_deb_config("/etc/network/interfaces")
+    ifaces = _parse_deb_config_data(...)
     config = ifaces_to_net_config_data(ifaces)
     state = parse_net_config_data(config)."""
     devs = {}
@@ -576,7 +564,9 @@ class Renderer(renderer.Renderer):
             netrules = subp.target_path(target, self.netrules_path)
             util.ensure_dir(os.path.dirname(netrules))
             util.write_file(
-                netrules, self._render_persistent_net(network_state)
+                netrules,
+                content=self._render_persistent_net(network_state),
+                preserve_mode=True,
             )
 
 
@@ -611,6 +601,3 @@ def available(target=None):
         return False
 
     return True
-
-
-# vi: ts=4 expandtab
