@@ -2,6 +2,7 @@
 import pytest
 
 from tests.integration_tests.instances import IntegrationInstance
+from tests.integration_tests.integration_settings import PLATFORM
 from tests.integration_tests.util import verify_clean_log
 
 USERDATA = """\
@@ -15,12 +16,9 @@ chef:
 
 
 @pytest.mark.adhoc  # Can't be regularly reaching out to chef install script
-@pytest.mark.ec2
-@pytest.mark.gce
-@pytest.mark.azure
-@pytest.mark.oci
-@pytest.mark.lxd_container
-@pytest.mark.lxd_vm
+@pytest.mark.skipif(
+    "openstack" == PLATFORM, reason="Firewall preventing openstack run"
+)
 @pytest.mark.user_data(USERDATA)
 def test_chef_license(client: IntegrationInstance):
     log = client.read_from_file("/var/log/cloud-init.log")
