@@ -16,14 +16,15 @@ KEEP_IMAGE = False
 RUN_UNSTABLE = False
 
 # One of:
-#  lxd_container
-#  lxd_vm
 #  azure
 #  ec2
 #  gce
 #  ibm
+#  lxd_container
+#  lxd_vm
 #  oci
 #  openstack
+#  qemu
 PLATFORM = "lxd_container"
 
 # The cloud-specific instance type to run. E.g., a1.medium on AWS
@@ -80,6 +81,13 @@ CLOUD_INIT_SOURCE = "NONE"
 COLLECT_LOGS = "ON_ERROR"
 LOCAL_LOG_PATH = "/tmp/cloud_init_test_logs"
 
+# We default our coverage to False because it involves modifying the
+# cloud-init systemd services, which is too intrusive of a change to
+# enable by default. If changed to true, the test directory corresponding
+# to the test run under LOCAL_LOG_PATH defined above will contain an
+# `html` directory with the coverage report.
+INCLUDE_COVERAGE = False
+
 ##################################################################
 # USER SETTINGS OVERRIDES
 ##################################################################
