@@ -2,6 +2,7 @@
 import pytest
 
 from tests.integration_tests.instances import IntegrationInstance
+from tests.integration_tests.integration_settings import PLATFORM
 from tests.integration_tests.util import (
     verify_clean_log,
     verify_ordered_items_in_text,
@@ -22,6 +23,9 @@ runcmd:
 """
 
 
+@pytest.mark.skipif(
+    PLATFORM == "qemu", reason="QEMU only supports #cloud-config header"
+)
 @pytest.mark.user_data(USER_DATA)
 def test_runcmd_with_variable_substitution(client: IntegrationInstance):
     """Test jinja substitution.
