@@ -3,6 +3,7 @@ from pathlib import Path
 import pytest
 
 from tests.integration_tests.instances import IntegrationInstance
+from tests.integration_tests.integration_settings import PLATFORM
 from tests.integration_tests.util import ASSETS_DIR, verify_clean_log
 
 PICKLE_PATH = Path("/var/lib/cloud/instance/obj.pkl")
@@ -41,12 +42,12 @@ def test_reboot_without_version_change(client: IntegrationInstance):
     )
 
 
-@pytest.mark.ec2
-@pytest.mark.gce
-@pytest.mark.oci
-@pytest.mark.openstack
-@pytest.mark.lxd_container
-@pytest.mark.lxd_vm
+@pytest.mark.skipif(
+    PLATFORM
+    not in ["ec2", "gce", "oci", "openstack", "lxd_container", "lxd_vm"],
+    reason=f"Test hasn't been tested on {PLATFORM}.",
+)
+# TODO: The below comment likely isn't true anymore
 # No Azure because the cache gets purged every reboot, so we'll never
 # get to the point where we need to purge cache due to version change
 def test_cache_purged_on_version_change(client: IntegrationInstance):
