@@ -9,9 +9,9 @@ from unittest import mock
 
 import pytest
 
+from cloudinit import subp
 from cloudinit.atomic_helper import write_json
 from cloudinit.cmd import status
-from cloudinit.cmd.status import UXAppStatus, _get_systemd_status
 from cloudinit.subp import SubpResult
 from cloudinit.util import ensure_file
 from tests.unittests.helpers import wrap_and_call
@@ -37,30 +37,42 @@ def config(tmpdir):
     )
 
 
+EXAMPLE_STATUS_RUNNING: Dict[str, Dict] = {
+    "v1": {
+        "datasource": None,
+        "init-local": {
+            "start": 1669231096.9621563,
+            "finished": None,
+            "errors": [],
+        },
+        "init": {"start": None, "finished": None, "errors": []},
+        "modules-config": {"start": None, "finished": None, "errors": []},
+        "modules-final": {"start": None, "finished": None, "errors": []},
+        "stage": "init-local",
+    }
+}
+
+
 class TestStatus:
     maxDiff = None
 
     @mock.patch(
-        M_PATH + "load_file",
-        return_value=(
-            '{"v1": {"datasource": null, "init": {"errors": [], "finished": '
-            'null, "start": null}, "init-local": {"errors": [], "finished": '
-            'null, "start": 1669231096.9621563}, "modules-config": '
-            '{"errors": [], "finished": null, "start": null},'
-            '"modules-final": {"errors": [], "finished": null, '
-            '"start": null}, "modules-init": {"errors": [], "finished": '
-            'null, "start": null}, "stage": "init-local"} }'
-        ),
+        M_PATH + "load_text_file",
+        return_value=json.dumps(EXAMPLE_STATUS_RUNNING),
     )
     @mock.patch(M_PATH + "os.path.exists", return_value=True)
+    @mock.patch(M_PATH + "is_running", return_value=True)
     @mock.patch(
         M_PATH + "get_bootstatus",
         return_value=(
-            status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+            status.EnabledStatus.ENABLED_BY_GENERATOR,
             "Cloud-init enabled by systemd cloud-init-generator",
         ),
     )
-    @mock.patch(f"{M_PATH}_get_systemd_status", return_value=None)
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=False,
+    )
     def test_get_status_details_ds_none(
         self,
         m_get_systemd_status,
@@ -72,14 +84,78 @@ class TestStatus:
         paths = mock.Mock()
         paths.run_dir = str(tmpdir)
         assert status.StatusDetails(
-            status.UXAppStatus.RUNNING,
-            status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+            status.RunningStatus.RUNNING,
+            status.ConditionStatus.PEACHY,
+            status.EnabledStatus.ENABLED_BY_GENERATOR,
             "Running in stage: init-local",
             [],
+            {},
             "Wed, 23 Nov 2022 19:18:16 +0000",
             None,  # datasource
+            {
+                "init-local": {
+                    "errors": [],
+                    "finished": None,
+                    "start": 1669231096.9621563,
+                },
+                "init": {"errors": [], "finished": None, "start": None},
+                "modules-config": {
+                    "errors": [],
+                    "finished": None,
+                    "start": None,
+                },
+                "modules-final": {
+                    "errors": [],
+                    "finished": None,
+                    "start": None,
+                },
+                "stage": "init-local",
+            },
         ) == status.get_status_details(paths)
 
+    @mock.patch(
+        M_PATH + "load_text_file",
+        return_value=json.dumps(EXAMPLE_STATUS_RUNNING),
+    )
+    @mock.patch(M_PATH + "os.path.exists", return_value=True)
+    @mock.patch(M_PATH + "is_running", return_value=True)
+    @mock.patch(
+        M_PATH + "get_bootstatus",
+        return_value=(
+            status.EnabledStatus.ENABLED_BY_GENERATOR,
+            "Cloud-init enabled by systemd cloud-init-generator",
+        ),
+    )
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=True,
+    )
+    @mock.patch(
+        f"{M_PATH}uses_systemd",
+        return_value=True,
+    )
+    def test_get_status_systemd_failure(
+        self,
+        m_uses_systemd,
+        m_systemd_status,
+        m_boot_status,
+        m_is_running,
+        m_p_exists,
+        m_load_json,
+        tmpdir,
+    ):
+        paths = mock.Mock()
+        paths.run_dir = str(tmpdir)
+        details = status.get_status_details(paths)
+        assert details.running_status == status.RunningStatus.DONE
+        assert details.condition_status == status.ConditionStatus.ERROR
+        assert details.description == "Failed due to systemd unit failure"
+        assert details.errors == [
+            "Failed due to systemd unit failure. Ensure all cloud-init "
+            "services are enabled, and check 'systemctl' or 'journalctl' "
+            "for more information."
+        ]
+
     @pytest.mark.parametrize(
         [
             "ensured_file",
@@ -95,7 +171,7 @@ class TestStatus:
                 lambda config: config.disable_file,
                 False,
                 "root=/dev/my-root not-important",
-                status.UXAppBootStatusCode.ENABLED_BY_SYSVINIT,
+                status.EnabledStatus.ENABLED_BY_SYSVINIT,
                 "expected enabled cloud-init on sysvinit",
                 "Cloud-init enabled on sysvinit",
                 id="false_on_sysvinit",
@@ -105,7 +181,7 @@ class TestStatus:
                 lambda config: config.disable_file,
                 True,
                 "root=/dev/my-root not-important",
-                status.UXAppBootStatusCode.DISABLED_BY_MARKER_FILE,
+                status.EnabledStatus.DISABLED_BY_MARKER_FILE,
                 "expected disabled cloud-init",
                 lambda config: f"Cloud-init disabled by {config.disable_file}",
                 id="true_on_disable_file",
@@ -115,7 +191,7 @@ class TestStatus:
                 lambda config: config.disable_file,
                 True,
                 "something cloud-init=enabled else",
-                status.UXAppBootStatusCode.ENABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
                 "expected enabled cloud-init",
                 "Cloud-init enabled by kernel command line cloud-init=enabled",
                 id="false_on_kernel_cmdline_enable",
@@ -125,7 +201,7 @@ class TestStatus:
                 None,
                 True,
                 "something cloud-init=disabled else",
-                status.UXAppBootStatusCode.DISABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.DISABLED_BY_KERNEL_CMDLINE,
                 "expected disabled cloud-init",
                 "Cloud-init disabled by kernel parameter cloud-init=disabled",
                 id="true_on_kernel_cmdline",
@@ -135,7 +211,7 @@ class TestStatus:
                 lambda config: os.path.join(config.paths.run_dir, "disabled"),
                 True,
                 "something",
-                status.UXAppBootStatusCode.DISABLED_BY_GENERATOR,
+                status.EnabledStatus.DISABLED_BY_GENERATOR,
                 "expected disabled cloud-init",
                 "Cloud-init disabled by cloud-init-generator",
                 id="true_when_generator_disables",
@@ -145,7 +221,7 @@ class TestStatus:
                 lambda config: os.path.join(config.paths.run_dir, "enabled"),
                 True,
                 "something ignored",
-                status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+                status.EnabledStatus.ENABLED_BY_GENERATOR,
                 "expected enabled cloud-init",
                 "Cloud-init enabled by systemd cloud-init-generator",
                 id="false_when_enabled_in_systemd",
@@ -161,19 +237,25 @@ class TestStatus:
         failure_msg: str,
         expected_reason: Union[str, Callable],
         config: Config,
+        mocker,
     ):
         if ensured_file is not None:
             ensure_file(ensured_file(config))
-        (code, reason) = wrap_and_call(
-            M_NAME,
-            {
-                "uses_systemd": uses_systemd,
-                "get_cmdline": get_cmdline,
-            },
-            status.get_bootstatus,
-            config.disable_file,
-            config.paths,
-        )
+        with mock.patch(
+            f"{M_PATH}subp.subp",
+            return_value=SubpResult(
+                """\
+LANG=en_US.UTF-8
+PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/snap/bin
+""",
+                stderr=None,
+            ),
+        ):
+            mocker.patch(f"{M_PATH}uses_systemd", return_value=uses_systemd)
+            mocker.patch(f"{M_PATH}get_cmdline", return_value=get_cmdline)
+            code, reason = status.get_bootstatus(
+                config.disable_file, config.paths, False
+            )
         assert code == expected_bootstatus, failure_msg
         if isinstance(expected_reason, str):
             assert reason == expected_reason
@@ -184,7 +266,7 @@ class TestStatus:
     def test_status_returns_not_run(
         self, m_read_cfg_paths, config: Config, capsys
     ):
-        """When status.json does not exist yet, return 'not run'."""
+        """When status.json does not exist yet, return 'not started'."""
         m_read_cfg_paths.return_value = config.paths
         assert not os.path.exists(
             config.status_file
@@ -192,14 +274,14 @@ class TestStatus:
         cmdargs = MyArgs(long=False, wait=False, format="tabular")
         retcode = wrap_and_call(
             M_NAME,
-            {"get_bootstatus": (status.UXAppBootStatusCode.UNKNOWN, "")},
+            {"get_bootstatus": (status.EnabledStatus.UNKNOWN, "")},
             status.handle_status_args,
             "ignored",
             cmdargs,
         )
         assert retcode == 0
         out, _err = capsys.readouterr()
-        assert out == "status: not run\n"
+        assert out == "status: not started\n"
 
     @mock.patch(M_PATH + "read_cfg_paths")
     def test_status_returns_disabled_long_on_presence_of_disable_file(
@@ -207,20 +289,14 @@ class TestStatus:
     ):
         """When cloudinit is disabled, return disabled reason."""
         m_read_cfg_paths.return_value = config.paths
-        checked_files = []
-
-        def fakeexists(filepath):
-            checked_files.append(filepath)
-            status_file = os.path.join(config.paths.run_dir, "status.json")
-            return bool(not filepath == status_file)
 
         cmdargs = MyArgs(long=True, wait=False, format="tabular")
         retcode = wrap_and_call(
             M_NAME,
             {
-                "os.path.exists": {"side_effect": fakeexists},
+                "os.path.exists": {"return_value": False},
                 "get_bootstatus": (
-                    status.UXAppBootStatusCode.DISABLED_BY_KERNEL_CMDLINE,
+                    status.EnabledStatus.DISABLED_BY_KERNEL_CMDLINE,
                     "disabled for some reason",
                 ),
             },
@@ -229,15 +305,14 @@ class TestStatus:
             cmdargs,
         )
         assert retcode == 0
-        assert checked_files == [
-            os.path.join(config.paths.run_dir, "status.json")
-        ]
         expected = dedent(
             """\
             status: disabled
+            extended_status: disabled
             boot_status_code: disabled-by-kernel-cmdline
-            detail:
-            disabled for some reason
+            detail: disabled for some reason
+            errors: []
+            recoverable_errors: {}
         """
         )
         out, _err = capsys.readouterr()
@@ -257,7 +332,7 @@ class TestStatus:
             # Report running when status.json exists but result.json does not.
             pytest.param(
                 None,
-                status.UXAppBootStatusCode.UNKNOWN,
+                status.EnabledStatus.UNKNOWN,
                 {},
                 lambda config: config.result_file,
                 MyArgs(long=False, wait=False, format="tabular"),
@@ -265,21 +340,10 @@ class TestStatus:
                 "status: running\n",
                 id="running_on_no_results_json",
             ),
-            # Report running when status exists with an unfinished stage.
-            pytest.param(
-                lambda config: config.result_file,
-                status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
-                {"v1": {"init": {"start": 1, "finished": None}}},
-                None,
-                MyArgs(long=False, wait=False, format="tabular"),
-                0,
-                "status: running\n",
-                id="running",
-            ),
             # Report done results.json exists no stages are unfinished.
             pytest.param(
                 lambda config: config.result_file,
-                status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+                status.EnabledStatus.ENABLED_BY_GENERATOR,
                 {
                     "v1": {
                         "stage": None,  # No current stage running
@@ -306,7 +370,7 @@ class TestStatus:
             # Long format of done status includes datasource info.
             pytest.param(
                 lambda config: config.result_file,
-                status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+                status.EnabledStatus.ENABLED_BY_GENERATOR,
                 {
                     "v1": {
                         "stage": None,
@@ -325,19 +389,20 @@ class TestStatus:
                 dedent(
                     """\
                     status: done
+                    extended_status: done
                     boot_status_code: enabled-by-generator
                     last_update: Thu, 01 Jan 1970 00:02:05 +0000
-                    detail:
-                    DataSourceNoCloud [seed=/var/.../seed/nocloud-net]\
-[dsmode=net]
-                    """
+                    detail: DataSourceNoCloud [seed=/var/.../seed/nocloud-net][dsmode=net]
+                    errors: []
+                    recoverable_errors: {}
+                    """  # noqa: E501
                 ),
                 id="returns_done_long",
             ),
             # Reports error when any stage has errors.
             pytest.param(
-                None,
-                status.UXAppBootStatusCode.ENABLED_BY_GENERATOR,
+                lambda config: config.result_file,
+                status.EnabledStatus.ENABLED_BY_GENERATOR,
                 {
                     "v1": {
                         "stage": None,
@@ -359,7 +424,7 @@ class TestStatus:
             # Long format of error status includes all error messages.
             pytest.param(
                 None,
-                status.UXAppBootStatusCode.ENABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
                 {
                     "v1": {
                         "stage": None,
@@ -385,21 +450,24 @@ class TestStatus:
                 1,
                 dedent(
                     """\
-                    status: error
-                    boot_status_code: enabled-by-kernel-cmdline
-                    last_update: Thu, 01 Jan 1970 00:02:05 +0000
-                    detail:
-                    error1
-                    error2
-                    error3
-                    """
+                status: error
+                extended_status: error - running
+                boot_status_code: enabled-by-kernel-cmdline
+                last_update: Thu, 01 Jan 1970 00:02:05 +0000
+                detail: DataSourceNoCloud [seed=/var/.../seed/nocloud-net][dsmode=net]
+                errors:
+                \t- error1
+                \t- error2
+                \t- error3
+                recoverable_errors: {}
+                """  # noqa: E501
                 ),
                 id="on_errors_long",
             ),
             # Long format reports the stage in which we are running.
             pytest.param(
                 None,
-                status.UXAppBootStatusCode.ENABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
                 {
                     "v1": {
                         "stage": "init",
@@ -413,17 +481,19 @@ class TestStatus:
                 dedent(
                     """\
                     status: running
+                    extended_status: running
                     boot_status_code: enabled-by-kernel-cmdline
                     last_update: Thu, 01 Jan 1970 00:02:04 +0000
-                    detail:
-                    Running in stage: init
+                    detail: Running in stage: init
+                    errors: []
+                    recoverable_errors: {}
                     """
                 ),
                 id="running_long_format",
             ),
             pytest.param(
                 None,
-                status.UXAppBootStatusCode.ENABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
                 {
                     "v1": {
                         "stage": "init",
@@ -437,20 +507,20 @@ class TestStatus:
                 dedent(
                     """\
                    ---
-                   _schema_version: '1'
                    boot_status_code: enabled-by-kernel-cmdline
                    datasource: ''
                    detail: 'Running in stage: init'
                    errors: []
+                   extended_status: running
+                   init:
+                       finished: null
+                       start: 124.456
+                   init-local:
+                       finished: 123.46
+                       start: 123.45
                    last_update: Thu, 01 Jan 1970 00:02:04 +0000
-                   schemas:
-                       '1':
-                           boot_status_code: enabled-by-kernel-cmdline
-                           datasource: ''
-                           detail: 'Running in stage: init'
-                           errors: []
-                           last_update: Thu, 01 Jan 1970 00:02:04 +0000
-                           status: running
+                   recoverable_errors: {}
+                   stage: init
                    status: running
                    ...
 
@@ -460,7 +530,7 @@ class TestStatus:
             ),
             pytest.param(
                 None,
-                status.UXAppBootStatusCode.ENABLED_BY_KERNEL_CMDLINE,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
                 {
                     "v1": {
                         "stage": "init",
@@ -472,36 +542,206 @@ class TestStatus:
                 MyArgs(long=False, wait=False, format="json"),
                 0,
                 {
-                    "_schema_version": "1",
                     "boot_status_code": "enabled-by-kernel-cmdline",
                     "datasource": "",
                     "detail": "Running in stage: init",
                     "errors": [],
-                    "last_update": "Thu, 01 Jan 1970 00:02:04 +0000",
-                    "schemas": {
-                        "1": {
-                            "boot_status_code": "enabled-by-kernel-cmdline",
-                            "datasource": "",
-                            "detail": "Running in stage: init",
-                            "errors": [],
-                            "last_update": "Thu, 01 Jan 1970 00:02:04 +0000",
-                            "status": "running",
-                        }
-                    },
                     "status": "running",
+                    "extended_status": "running",
+                    "init": {"finished": None, "start": 124.456},
+                    "init-local": {"finished": 123.46, "start": 123.45},
+                    "last_update": "Thu, 01 Jan 1970 00:02:04 +0000",
+                    "recoverable_errors": {},
+                    "stage": "init",
                 },
                 id="running_json_format",
             ),
+            pytest.param(
+                None,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
+                {
+                    "v1": {
+                        "stage": None,
+                        "datasource": (
+                            "DataSourceNoCloud "
+                            "[seed=/var/.../seed/nocloud-net]"
+                            "[dsmode=net]"
+                        ),
+                        "init": {
+                            "errors": ["error1"],
+                            "start": 124.567,
+                            "finished": 125.678,
+                        },
+                        "init-local": {
+                            "errors": ["error2", "error3"],
+                            "start": 123.45,
+                            "finished": 123.46,
+                        },
+                    }
+                },
+                None,
+                MyArgs(long=False, wait=False, format="json"),
+                1,
+                {
+                    "boot_status_code": "enabled-by-kernel-cmdline",
+                    "datasource": "nocloud",
+                    "detail": (
+                        "DataSourceNoCloud [seed=/var/.../seed/"
+                        "nocloud-net][dsmode=net]"
+                    ),
+                    "errors": ["error1", "error2", "error3"],
+                    "status": "error",
+                    "extended_status": "error - running",
+                    "init": {
+                        "finished": 125.678,
+                        "start": 124.567,
+                        "errors": ["error1"],
+                    },
+                    "init-local": {
+                        "finished": 123.46,
+                        "start": 123.45,
+                        "errors": ["error2", "error3"],
+                    },
+                    "last_update": "Thu, 01 Jan 1970 00:02:05 +0000",
+                    "recoverable_errors": {},
+                    "stage": None,
+                },
+                id="running_json_format_with_errors",
+            ),
+            pytest.param(
+                lambda config: config.result_file,
+                status.EnabledStatus.ENABLED_BY_KERNEL_CMDLINE,
+                {
+                    "v1": {
+                        "stage": None,
+                        "datasource": (
+                            "DataSourceNoCloud "
+                            "[seed=/var/.../seed/nocloud-net]"
+                            "[dsmode=net]"
+                        ),
+                        "modules-final": {
+                            "errors": [],
+                            "recoverable_errors": {
+                                "DEPRECATED": [
+                                    (
+                                        "don't try to open the hatch "
+                                        "or we'll all be soup"
+                                    )
+                                ]
+                            },
+                            "start": 127.567,
+                            "finished": 128.678,
+                        },
+                        "modules-config": {
+                            "errors": [],
+                            "recoverable_errors": {
+                                "CRITICAL": ["Power lost! Prepare to"]
+                            },
+                            "start": 125.567,
+                            "finished": 126.678,
+                        },
+                        "init": {
+                            "errors": [],
+                            "recoverable_errors": {
+                                "WARNINGS": [
+                                    "the prime omega transfuser borkeded!"
+                                ]
+                            },
+                            "start": 124.567,
+                            "finished": 125.678,
+                        },
+                        "init-local": {
+                            "errors": [],
+                            "recoverable_errors": {
+                                "ERROR": [
+                                    "the ion field reactor just transmutated"
+                                ]
+                            },
+                            "start": 123.45,
+                            "finished": 123.46,
+                        },
+                    }
+                },
+                None,
+                MyArgs(long=False, wait=False, format="json"),
+                2,
+                {
+                    "boot_status_code": "enabled-by-kernel-cmdline",
+                    "datasource": "nocloud",
+                    "detail": (
+                        "DataSourceNoCloud [seed=/var/.../"
+                        "seed/nocloud-net][dsmode=net]"
+                    ),
+                    "errors": [],
+                    "status": "done",
+                    "extended_status": "degraded done",
+                    "modules-final": {
+                        "errors": [],
+                        "recoverable_errors": {
+                            "DEPRECATED": [
+                                (
+                                    "don't try to open the "
+                                    "hatch or we'll all be soup"
+                                )
+                            ]
+                        },
+                        "start": 127.567,
+                        "finished": 128.678,
+                    },
+                    "modules-config": {
+                        "errors": [],
+                        "recoverable_errors": {
+                            "CRITICAL": ["Power lost! Prepare to"]
+                        },
+                        "start": 125.567,
+                        "finished": 126.678,
+                    },
+                    "init": {
+                        "errors": [],
+                        "recoverable_errors": {
+                            "WARNINGS": [
+                                "the prime omega transfuser borkeded!"
+                            ]
+                        },
+                        "start": 124.567,
+                        "finished": 125.678,
+                    },
+                    "init-local": {
+                        "errors": [],
+                        "recoverable_errors": {
+                            "ERROR": [
+                                "the ion field reactor just transmutated"
+                            ]
+                        },
+                        "start": 123.45,
+                        "finished": 123.46,
+                    },
+                    "last_update": "Thu, 01 Jan 1970 00:02:08 +0000",
+                    "recoverable_errors": {
+                        "ERROR": ["the ion field reactor just transmutated"],
+                        "WARNINGS": ["the prime omega transfuser borkeded!"],
+                        "CRITICAL": ["Power lost! Prepare to"],
+                        "DEPRECATED": [
+                            "don't try to open the hatch or we'll all be soup"
+                        ],
+                    },
+                    "stage": None,
+                },
+                id="running_json_format_with_recoverable_errors",
+            ),
         ],
     )
     @mock.patch(M_PATH + "read_cfg_paths")
-    @mock.patch(f"{M_PATH}_get_systemd_status", return_value=None)
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=None,
+    )
     def test_status_output(
         self,
         m_get_systemd_status,
         m_read_cfg_paths,
         ensured_file: Optional[Callable],
-        bootstatus: status.UXAppBootStatusCode,
+        bootstatus: status.EnabledStatus,
         status_content: Dict,
         assert_file,
         cmdargs: MyArgs,
@@ -536,7 +776,10 @@ class TestStatus:
             assert out == expected_status
 
     @mock.patch(M_PATH + "read_cfg_paths")
-    @mock.patch(f"{M_PATH}_get_systemd_status", return_value=None)
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=None,
+    )
     def test_status_wait_blocks_until_done(
         self, m_get_systemd_status, m_read_cfg_paths, config: Config, capsys
     ):
@@ -575,7 +818,7 @@ class TestStatus:
             M_NAME,
             {
                 "sleep": {"side_effect": fake_sleep},
-                "get_bootstatus": (status.UXAppBootStatusCode.UNKNOWN, ""),
+                "get_bootstatus": (status.EnabledStatus.UNKNOWN, ""),
             },
             status.handle_status_args,
             "ignored",
@@ -587,7 +830,10 @@ class TestStatus:
         assert out == "....\nstatus: done\n"
 
     @mock.patch(M_PATH + "read_cfg_paths")
-    @mock.patch(f"{M_PATH}_get_systemd_status", return_value=None)
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=None,
+    )
     def test_status_wait_blocks_until_error(
         self, m_get_systemd_status, m_read_cfg_paths, config: Config, capsys
     ):
@@ -622,13 +868,14 @@ class TestStatus:
                 write_json(config.status_file, running_json)
             elif sleep_calls == 3:
                 write_json(config.status_file, error_json)
+                write_json(config.result_file, "{}")
 
         cmdargs = MyArgs(long=False, wait=True, format="tabular")
         retcode = wrap_and_call(
             M_NAME,
             {
                 "sleep": {"side_effect": fake_sleep},
-                "get_bootstatus": (status.UXAppBootStatusCode.UNKNOWN, ""),
+                "get_bootstatus": (status.EnabledStatus.UNKNOWN, ""),
             },
             status.handle_status_args,
             "ignored",
@@ -640,7 +887,10 @@ class TestStatus:
         assert out == "....\nstatus: error\n"
 
     @mock.patch(M_PATH + "read_cfg_paths")
-    @mock.patch(f"{M_PATH}_get_systemd_status", return_value=None)
+    @mock.patch(
+        f"{M_PATH}systemd_failed",
+        return_value=None,
+    )
     def test_status_main(
         self, m_get_systemd_status, m_read_cfg_paths, config: Config, capsys
     ):
@@ -655,7 +905,7 @@ class TestStatus:
                 M_NAME,
                 {
                     "sys.argv": {"new": ["status"]},
-                    "get_bootstatus": (status.UXAppBootStatusCode.UNKNOWN, ""),
+                    "get_bootstatus": (status.EnabledStatus.UNKNOWN, ""),
                 },
                 status.main,
             )
@@ -664,45 +914,61 @@ class TestStatus:
         assert out == "status: running\n"
 
 
-class TestSystemdStatusDetails:
+class TestSystemdFailed:
+    @pytest.fixture(autouse=True)
+    def common_mocks(self, mocker):
+        mocker.patch("cloudinit.cmd.status.sleep")
+        yield
+
     @pytest.mark.parametrize(
-        ["active_state", "unit_file_state", "sub_state", "main_pid", "status"],
+        [
+            "active_state",
+            "unit_file_state",
+            "sub_state",
+            "main_pid",
+            "expected_failed",
+        ],
         [
             # To cut down on the combination of states, I'm grouping
             # enabled, enabled-runtime, and static into an "enabled" state
             # and everything else functionally disabled.
             # Additionally, SubStates are undocumented and may mean something
-            # different depending on the ActiveState they are mapped too.
+            # different depending on the ActiveState they are mapped to.
             # Because of this I'm only testing SubState combinations seen
             # in real-world testing (or using "any" string if we dont care).
-            ("activating", "enabled", "start", "123", UXAppStatus.RUNNING),
-            ("activating", "enabled", "start", "123", UXAppStatus.RUNNING),
-            ("active", "enabled-runtime", "exited", "0", None),
-            ("active", "enabled", "exited", "0", None),
-            ("active", "enabled", "running", "345", UXAppStatus.RUNNING),
-            ("active", "enabled", "running", "0", None),
-            # Dead doesn't mean exited here. It means not run yet.
-            ("inactive", "static", "dead", "123", UXAppStatus.RUNNING),
-            ("reloading", "enabled", "start", "123", UXAppStatus.RUNNING),
+            ("activating", "enabled", "start", "123", False),
+            ("activating", "enabled", "start", "123", False),
+            ("active", "enabled-runtime", "exited", "0", False),
+            ("active", "enabled", "exited", "0", False),
+            ("active", "enabled", "running", "345", False),
+            ("active", "enabled", "running", "0", False),
+            # Dead doesn't mean exited here. It means not started yet.
+            ("inactive", "static", "dead", "123", False),
+            ("reloading", "enabled", "start", "123", False),
             (
                 "deactivating",
                 "enabled-runtime",
                 "any",
                 "123",
-                UXAppStatus.RUNNING,
+                False,
             ),
-            ("failed", "static", "failed", "0", UXAppStatus.ERROR),
+            ("failed", "static", "failed", "0", True),
             # Try previous combinations again with "not enabled" states
-            ("activating", "linked", "start", "0", UXAppStatus.ERROR),
-            ("active", "linked-runtime", "exited", "0", UXAppStatus.ERROR),
-            ("inactive", "masked", "dead", "0", UXAppStatus.ERROR),
-            ("reloading", "masked-runtime", "start", "0", UXAppStatus.ERROR),
-            ("deactivating", "disabled", "any", "0", UXAppStatus.ERROR),
-            ("failed", "invalid", "failed", "0", UXAppStatus.ERROR),
+            ("activating", "linked", "start", "0", True),
+            ("active", "linked-runtime", "exited", "0", True),
+            ("inactive", "masked", "dead", "0", True),
+            ("reloading", "masked-runtime", "start", "0", True),
+            ("deactivating", "disabled", "any", "0", True),
+            ("failed", "invalid", "failed", "0", True),
         ],
     )
-    def test_get_systemd_status(
-        self, active_state, unit_file_state, sub_state, main_pid, status
+    def test_systemd_failed(
+        self,
+        active_state,
+        unit_file_state,
+        sub_state,
+        main_pid,
+        expected_failed,
     ):
         with mock.patch(
             f"{M_PATH}subp.subp",
@@ -714,4 +980,84 @@ class TestSystemdStatusDetails:
                 stderr=None,
             ),
         ):
-            assert _get_systemd_status() == status
+            assert status.systemd_failed(wait=False) == expected_failed
+
+    def test_retry(self, mocker, capsys):
+        m_subp = mocker.patch(
+            f"{M_PATH}subp.subp",
+            side_effect=[
+                subp.ProcessExecutionError(
+                    "Message recipient disconnected from message bus without"
+                    " replying"
+                ),
+                subp.ProcessExecutionError(
+                    "Message recipient disconnected from message bus without"
+                    " replying"
+                ),
+                SubpResult(
+                    "ActiveState=activating\nUnitFileState=enabled\n"
+                    "SubState=start\nMainPID=123\n",
+                    stderr=None,
+                ),
+            ],
+        )
+        assert status.systemd_failed(wait=True) is False
+        assert 3 == m_subp.call_count
+        assert "Failed to get status" not in capsys.readouterr().err
+
+    def test_retry_no_wait(self, mocker, capsys):
+        m_subp = mocker.patch(
+            f"{M_PATH}subp.subp",
+            side_effect=subp.ProcessExecutionError(
+                stderr=(
+                    "Message recipient disconnected from message bus without "
+                    "replying"
+                ),
+            ),
+        )
+        mocker.patch("time.time", side_effect=[1, 2, 50])
+        assert status.systemd_failed(wait=False) is False
+        assert 1 == m_subp.call_count
+        assert (
+            "Failed to get status from systemd. "
+            "Cloud-init status may be inaccurate. "
+            "Error from systemctl: Message recipient disconnected from "
+            "message bus without replying"
+        ) in capsys.readouterr().err
+
+
+class TestQuerySystemctl:
+    def test_query_systemctl(self, mocker):
+        m_subp = mocker.patch(
+            f"{M_PATH}subp.subp",
+            return_value=SubpResult(stdout="hello", stderr=None),
+        )
+        assert status.query_systemctl(["some", "args"], wait=False) == "hello"
+        m_subp.assert_called_once_with(["systemctl", "some", "args"])
+
+    def test_query_systemctl_with_exception(self, mocker, capsys):
+        m_subp = mocker.patch(
+            f"{M_PATH}subp.subp",
+            side_effect=subp.ProcessExecutionError(
+                "Message recipient disconnected", stderr="oh noes!"
+            ),
+        )
+        with pytest.raises(subp.ProcessExecutionError):
+            status.query_systemctl(["some", "args"], wait=False)
+        m_subp.assert_called_once_with(["systemctl", "some", "args"])
+
+    def test_query_systemctl_wait_with_exception(self, mocker):
+        m_sleep = mocker.patch(f"{M_PATH}sleep")
+        m_subp = mocker.patch(
+            f"{M_PATH}subp.subp",
+            side_effect=[
+                subp.ProcessExecutionError("Message recipient disconnected"),
+                subp.ProcessExecutionError("Message recipient disconnected"),
+                subp.ProcessExecutionError("Message recipient disconnected"),
+                SubpResult(stdout="hello", stderr=None),
+            ],
+        )
+
+        assert status.query_systemctl(["some", "args"], wait=True) == "hello"
+        assert m_subp.call_count == 4
+        assert m_sleep.call_count == 3
