@@ -10,7 +10,7 @@ import textwrap
 
 import pytest
 
-from cloudinit import cloud, helpers, util
+from cloudinit import cloud, helpers, temp_utils, util
 from cloudinit.config import cc_apk_configure
 from cloudinit.config.schema import (
     SchemaValidationError,
@@ -18,6 +18,7 @@ from cloudinit.config.schema import (
     validate_cloudconfig_schema,
 )
 from tests.unittests.helpers import (
+    SCHEMA_EMPTY_ERROR,
     FilesystemMockingTestCase,
     mock,
     skipUnlessJsonSchema,
@@ -59,6 +60,11 @@ class TestConfig(FilesystemMockingTestCase):
         self.name = "apk_configure"
         self.cloud = cloud.Cloud(None, self.paths, None, None, None)
         self.args = []
+        temp_utils._TMPDIR = self.new_root
+
+    def tearDown(self):
+        super().tearDown()
+        temp_utils._TMPDIR = None
 
     @mock.patch(CC_APK + "._write_repositories_file")
     def test_no_repo_settings(self, m_write_repos):
@@ -107,7 +113,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
     def test_main_and_community_repos(self):
         """
@@ -142,7 +148,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
     def test_main_community_testing_repos(self):
         """
@@ -182,7 +188,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
     def test_edge_main_community_testing_repos(self):
         """
@@ -219,7 +225,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
     def test_main_community_testing_local_repos(self):
         """
@@ -266,7 +272,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
     def test_edge_main_community_testing_local_repos(self):
         """
@@ -310,7 +316,7 @@ class TestConfig(FilesystemMockingTestCase):
             )
         )
 
-        self.assertEqual(expected_content, util.load_file(REPO_FILE))
+        self.assertEqual(expected_content, util.load_text_file(REPO_FILE))
 
 
 class TestApkConfigureSchema:
@@ -355,7 +361,7 @@ class TestApkConfigureSchema:
             (
                 {"apk_repos": {"alpine_repo": {}}},
                 "apk_repos.alpine_repo: 'version' is a required property,"
-                " apk_repos.alpine_repo: {} does not have enough properties",
+                f" apk_repos.alpine_repo: {{}} {SCHEMA_EMPTY_ERROR}",
             ),
             (
                 {"apk_repos": {"alpine_repo": True}},
@@ -368,7 +374,7 @@ class TestApkConfigureSchema:
             ),
             (
                 {"apk_repos": {}},
-                "apk_repos: {} does not have enough properties",
+                f"apk_repos: {{}} {SCHEMA_EMPTY_ERROR}",
             ),
             (
                 {"apk_repos": {"local_repo_base_url": None}},
@@ -384,6 +390,3 @@ class TestApkConfigureSchema:
         else:
             with pytest.raises(SchemaValidationError, match=error_msg):
                 validate_cloudconfig_schema(config, schema, strict=True)
-
-
-# vi: ts=4 expandtab
