@@ -8,16 +8,18 @@ from unittest import mock
 
 import pytest
 
-from cloudinit import distros, helpers
-from cloudinit import log as logger
-from cloudinit import subp, util
+from cloudinit import distros, helpers, subp, util
 from cloudinit.config import cc_ca_certs
 from cloudinit.config.schema import (
     SchemaValidationError,
     get_schema,
     validate_cloudconfig_schema,
 )
-from tests.unittests.helpers import TestCase, skipUnlessJsonSchema
+from tests.unittests.helpers import (
+    SCHEMA_EMPTY_ERROR,
+    TestCase,
+    skipUnlessJsonSchema,
+)
 from tests.unittests.util import get_cloud
 
 
@@ -327,7 +329,7 @@ class TestRemoveDefaultCaCerts(TestCase):
                 )
                 mock_load = mocks.enter_context(
                     mock.patch.object(
-                        util, "load_file", return_value=ca_certs_content
+                        util, "load_text_file", return_value=ca_certs_content
                     )
                 )
                 mock_subp = mocks.enter_context(
@@ -339,7 +341,7 @@ class TestRemoveDefaultCaCerts(TestCase):
 
                 cc_ca_certs.disable_default_ca_certs(distro_name, conf)
 
-                if distro_name == "rhel":
+                if distro_name in ["rhel", "photon"]:
                     mock_delete.assert_has_calls(
                         [
                             mock.call(conf["ca_cert_path"]),
@@ -356,8 +358,10 @@ class TestRemoveDefaultCaCerts(TestCase):
                     if distro_name in ["debian", "ubuntu"]:
                         mock_subp.assert_called_once_with(
                             ("debconf-set-selections", "-"),
-                            "ca-certificates ca-certificates/trust_new_crts"
-                            " select no",
+                            data=(
+                                "ca-certificates ca-certificates/"
+                                "trust_new_crts select no"
+                            ),
                         )
                     else:
                         assert mock_subp.call_count == 0
@@ -396,7 +400,7 @@ class TestCACertsSchema:
             ),
             (
                 {"ca_certs": {}},
-                re.escape("ca_certs: {} does not have enough properties"),
+                re.escape("ca_certs: {} ") + SCHEMA_EMPTY_ERROR,
             ),
             (
                 {"ca_certs": {"boguskey": 1}},
@@ -415,7 +419,7 @@ class TestCACertsSchema:
             ),
             (
                 {"ca_certs": {"trusted": []}},
-                re.escape("ca_certs.trusted: [] is too short"),
+                re.escape("ca_certs.trusted: [] ") + SCHEMA_EMPTY_ERROR,
             ),
         ),
     )
@@ -433,7 +437,6 @@ class TestCACertsSchema:
     @mock.patch.object(cc_ca_certs, "update_ca_certs")
     def test_deprecate_key_warnings(self, update_ca_certs, caplog):
         """Assert warnings are logged for deprecated keys."""
-        logger.setupLogging()
         cloud = get_cloud("ubuntu")
         cc_ca_certs.handle(
             "IGNORE", {"ca-certs": {"remove-defaults": False}}, cloud, []
@@ -444,7 +447,7 @@ class TestCACertsSchema:
         ]
         for warning in expected_warnings:
             assert warning in caplog.text
-            assert "DEPRECAT" in caplog.text
+            assert "deprecat" in caplog.text
         assert 1 == update_ca_certs.call_count
 
     @mock.patch.object(cc_ca_certs, "update_ca_certs")
