@@ -583,9 +583,15 @@ class TestMountsSchema:
         "config, error_msg",
         [
             # We expect to see one mount if provided in user-data.
-            ({"mounts": []}, re.escape("mounts: [] is too short")),
+            (
+                {"mounts": []},
+                re.escape("mounts: [] ") + test_helpers.SCHEMA_EMPTY_ERROR,
+            ),
             # Disallow less than 1 item per mount entry
-            ({"mounts": [[]]}, re.escape("mounts.0: [] is too short")),
+            (
+                {"mounts": [[]]},
+                re.escape("mounts.0: [] ") + test_helpers.SCHEMA_EMPTY_ERROR,
+            ),
             # Disallow more than 6 items per mount entry
             ({"mounts": [["1"] * 7]}, "mounts.0:.* is too long"),
             # Disallow mount_default_fields will anything other than 6 items
@@ -612,13 +618,16 @@ class TestMountsSchema:
             ({"swap": {"size": "1.5MB"}}, "swap.size:"),
             (
                 {"swap": {"maxsize": "1.5MT"}},
-                "swap.maxsize: '1.5MT' is not valid",
+                re.escape("swap.maxsize: '1.5MT' does not match '^([0-9]+)?"),
             ),
             (
                 {"swap": {"maxsize": "..5T"}},
-                "swap.maxsize: '..5T' is not valid",
+                re.escape("swap.maxsize: '..5T' does not match '^([0-9]+)?"),
+            ),
+            (
+                {"swap": {"size": "K"}},
+                "swap.size: 'K' is not of type 'integer'",
             ),
-            ({"swap": {"size": "K"}}, "swap.size: 'K' is not valid"),
         ],
     )
     @test_helpers.skipUnlessJsonSchema()
