@@ -11,6 +11,7 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
+from cloudinit.distros import PackageInstallerError
 from cloudinit.subp import ProcessExecutionError
 from tests.unittests.helpers import CiTestCase, mock, skipUnlessJsonSchema
 from tests.unittests.util import get_cloud
@@ -115,7 +116,7 @@ class TestPuppetHandle(CiTestCase):
         cfg = {"puppet": {}}
         cc_puppet.handle("notimportant", cfg, self.cloud, None)
         self.assertEqual(
-            [mock.call(("puppet-agent", None))],
+            [mock.call(["puppet-agent"])],
             self.cloud.distro.install_packages.call_args_list,
         )
 
@@ -127,7 +128,7 @@ class TestPuppetHandle(CiTestCase):
         cfg = {"puppet": {"install": True}}
         cc_puppet.handle("notimportant", cfg, self.cloud, None)
         self.assertIn(
-            [mock.call(("puppet-agent", None))],
+            [mock.call(["puppet-agent"])],
             self.cloud.distro.install_packages.call_args_list,
         )
 
@@ -236,7 +237,7 @@ class TestPuppetHandle(CiTestCase):
         cfg = {"puppet": {"version": "3.8"}}
         cc_puppet.handle("notimportant", cfg, self.cloud, None)
         self.assertEqual(
-            [mock.call(("puppet-agent", "3.8"))],
+            [mock.call([["puppet-agent", "3.8"]])],
             self.cloud.distro.install_packages.call_args_list,
         )
 
@@ -260,7 +261,7 @@ class TestPuppetHandle(CiTestCase):
         util.write_file(self.conf, "[agent]\nserver = origpuppet\nother = 3")
         self.cloud.distro = mock.MagicMock()
         cc_puppet.handle("notimportant", cfg, self.cloud, None)
-        content = util.load_file(self.conf)
+        content = util.load_text_file(self.conf)
         expected = "[agent]\nserver = puppetserver.example.org\nother = 3\n\n"
         self.assertEqual(expected, content)
 
@@ -297,7 +298,7 @@ class TestPuppetHandle(CiTestCase):
             }
         }
         cc_puppet.handle("notimportant", cfg, self.cloud, None)
-        content = util.load_file(self.csr_attributes_path)
+        content = util.load_text_file(self.csr_attributes_path)
         expected = textwrap.dedent(
             """\
             custom_attributes:
@@ -407,7 +408,7 @@ class TestPuppetHandle(CiTestCase):
             "tests.unittests.util.MockDistro.install_packages"
         ) as install_pkg:
             # puppet-agent not installed, but puppet is
-            install_pkg.side_effect = (ProcessExecutionError, 0)
+            install_pkg.side_effect = (PackageInstallerError, 0)
 
             cc_puppet.handle("notimportant", cfg, self.cloud, None)
             expected_calls = [
