@@ -43,7 +43,7 @@ class TestConfig(helpers.FilesystemMockingTestCase):
         self.patchUtils(self.tmp)
         cc_yum_add_repo.handle("yum_add_repo", cfg, None, [])
         self.assertRaises(
-            IOError, util.load_file, "/etc/yum.repos.d/epel_testing.repo"
+            IOError, util.load_text_file, "/etc/yum.repos.d/epel_testing.repo"
         )
 
     def test_write_config(self):
@@ -62,7 +62,7 @@ class TestConfig(helpers.FilesystemMockingTestCase):
         self.patchUtils(self.tmp)
         self.patchOS(self.tmp)
         cc_yum_add_repo.handle("yum_add_repo", cfg, None, [])
-        contents = util.load_file("/etc/yum.repos.d/epel-testing.repo")
+        contents = util.load_text_file("/etc/yum.repos.d/epel-testing.repo")
         parser = configparser.ConfigParser()
         parser.read_string(contents)
         expected = {
@@ -102,7 +102,9 @@ class TestConfig(helpers.FilesystemMockingTestCase):
         }
         self.patchUtils(self.tmp)
         cc_yum_add_repo.handle("yum_add_repo", cfg, None, [])
-        contents = util.load_file("/etc/yum.repos.d/puppetlabs-products.repo")
+        contents = util.load_text_file(
+            "/etc/yum.repos.d/puppetlabs-products.repo"
+        )
         parser = configparser.ConfigParser()
         parser.read_string(contents)
         expected = {
@@ -139,7 +141,7 @@ class TestAddYumRepoSchema:
             ),
             (
                 {"yum_repos": {}},
-                re.escape("yum_repos: {} does not have enough properties"),
+                re.escape("yum_repos: {} ") + helpers.SCHEMA_EMPTY_ERROR,
             ),
             # baseurl required
             (
@@ -178,6 +180,3 @@ class TestAddYumRepoSchema:
         else:
             with pytest.raises(SchemaValidationError, match=error_msg):
                 validate_cloudconfig_schema(config, get_schema(), strict=True)
-
-
-# vi: ts=4 expandtab
