@@ -12,7 +12,6 @@ import tempfile
 import time
 import unittest
 from contextlib import ExitStack, contextmanager
-from pathlib import Path
 from typing import ClassVar, List, Union
 from unittest import mock
 from unittest.util import strclass
@@ -20,8 +19,7 @@ from urllib.parse import urlsplit, urlunsplit
 
 import responses
 
-import cloudinit
-from cloudinit import cloud, distros
+from cloudinit import atomic_helper, cloud, distros
 from cloudinit import helpers as ch
 from cloudinit import subp, util
 from cloudinit.config.schema import (
@@ -30,6 +28,7 @@ from cloudinit.config.schema import (
 )
 from cloudinit.sources import DataSourceNone
 from cloudinit.templater import JINJA_AVAILABLE
+from tests.helpers import cloud_init_project_dir
 from tests.hypothesis_jsonschema import HAS_HYPOTHESIS_JSONSCHEMA
 
 _real_subp = subp.subp
@@ -39,6 +38,22 @@ SkipTest = unittest.SkipTest
 skipIf = unittest.skipIf
 
 
+try:
+    import apt_pkg  # type: ignore # noqa: F401
+
+    HAS_APT_PKG = True
+except ImportError:
+    HAS_APT_PKG = False
+
+
+# Used by tests to verify the error message when a jsonschema structure
+# is empty but should not be.
+# Version 4.20.0 of jsonschema changed the error messages for empty structures.
+SCHEMA_EMPTY_ERROR = (
+    "(is too short|should be non-empty|does not have enough properties)"
+)
+
+
 # Makes the old path start
 # with new base instead of whatever
 # it previously had
@@ -61,7 +76,7 @@ def retarget_many_wrapper(new_base, am, old_func):
         nam = am
         if am == -1:
             nam = len(n_args)
-        for i in range(0, nam):
+        for i in range(nam):
             path = args[i]
             # patchOS() wraps various os and os.path functions, however in
             # Python 3 some of these now accept file-descriptors (integers).
@@ -145,6 +160,7 @@ class CiTestCase(TestCase):
             handler.setFormatter(formatter)
             self.old_handlers = self.logger.handlers
             self.logger.handlers = [handler]
+            self.old_level = logging.root.level
         if self.allowed_subp is True:
             subp.subp = _real_subp
         else:
@@ -186,7 +202,7 @@ class CiTestCase(TestCase):
         if self.with_logs:
             # Remove the handler we setup
             logging.getLogger().handlers = self.old_handlers
-            logging.getLogger().setLevel(logging.NOTSET)
+            logging.getLogger().setLevel(self.old_level)
         subp.subp = _real_subp
         super(CiTestCase, self).tearDown()
 
@@ -268,8 +284,8 @@ class FilesystemMockingTestCase(ResourceUsingTestCase):
             make_path = rebase_path(real_path[len(real_root) :], target_root)
             util.ensure_dir(make_path)
             for f in filenames:
-                real_path = util.abs_join(real_path, f)
-                make_path = util.abs_join(make_path, f)
+                real_path = os.path.abspath(os.path.join(real_path, f))
+                make_path = os.path.abspath(os.path.join(make_path, f))
                 shutil.copy(real_path, make_path)
 
     def patchUtils(self, new_root):
@@ -277,7 +293,8 @@ class FilesystemMockingTestCase(ResourceUsingTestCase):
             util: [
                 ("write_file", 1),
                 ("append_file", 1),
-                ("load_file", 1),
+                ("load_binary_file", 1),
+                ("load_text_file", 1),
                 ("ensure_dir", 1),
                 ("chmod", 1),
                 ("delete_dir_contents", 1),
@@ -285,6 +302,9 @@ class FilesystemMockingTestCase(ResourceUsingTestCase):
                 ("sym_link", -1),
                 ("copy", -1),
             ],
+            atomic_helper: [
+                ("write_json", 1),
+            ],
         }
         for (mod, funcs) in patch_funcs.items():
             for (f, am) in funcs:
@@ -470,7 +490,7 @@ def dir2dict(startdir, prefix=None):
         for fname in files:
             fpath = os.path.join(root, fname)
             key = fpath[len(prefix) :]
-            flist[key] = util.load_file(fpath)
+            flist[key] = util.load_text_file(fpath)
     return flist
 
 
@@ -519,6 +539,13 @@ def readResource(name, mode="r"):
         return fh.read()
 
 
+def skipIfAptPkg():
+    return skipIf(
+        HAS_APT_PKG,
+        "No python-apt dependency present.",
+    )
+
+
 try:
     import jsonschema
 
@@ -578,24 +605,6 @@ if not hasattr(mock.Mock, "assert_not_called"):
     mock.Mock.assert_not_called = __mock_assert_not_called  # type: ignore
 
 
-def get_top_level_dir() -> Path:
-    """Return the absolute path to the top cloudinit project directory
-
-    @return Path('<top-cloudinit-dir>')
-    """
-    return Path(cloudinit.__file__).parent.parent.resolve()
-
-
-def cloud_init_project_dir(sub_path: str) -> str:
-    """Get a path within the cloudinit project directory
-
-    @return str of the combined path
-
-    Example: cloud_init_project_dir("my/path") -> "/path/to/cloud-init/my/path"
-    """
-    return str(get_top_level_dir() / sub_path)
-
-
 @contextmanager
 def does_not_raise():
     """Context manager to parametrize tests raising and not raising exceptions
@@ -619,6 +628,3 @@ def does_not_raise():
 
     """
     yield
-
-
-# vi: ts=4 expandtab
