@@ -8,6 +8,7 @@ from unittest import mock
 
 import pytest
 import requests
+import responses
 
 from cloudinit.sources.azure import imds
 from cloudinit.url_helper import UrlError, readurl
@@ -23,6 +24,24 @@ class StringMatch:
     def __eq__(self, other) -> bool:
         return bool(re.match("^" + self.regex + "$", other))
 
+    def __repr__(self) -> str:
+        return repr(self.regex)
+
+
+@pytest.fixture(autouse=True)
+def caplog(caplog):
+    # Ensure caplog is set to debug.
+    caplog.set_level(logging.DEBUG)
+    yield caplog
+
+
+@pytest.fixture
+def mock_requests():
+    with responses.RequestsMock(
+        assert_all_requests_are_fired=True
+    ) as response:
+        yield response
+
 
 @pytest.fixture
 def wrapped_readurl():
@@ -67,6 +86,49 @@ def fake_http_error_for_code(status_code: int):
     )
 
 
+REQUESTS_CONNECTION_ERROR = requests.ConnectionError("Fake connection error")
+
+REQUESTS_TIMEOUT_ERROR = requests.Timeout("Fake connection timeout")
+
+
+def add_errors_to_mock_requests(mock_requests, errors, url):
+    def callback_connection_error(request):
+        raise REQUESTS_CONNECTION_ERROR
+
+    def callback_timeout(request):
+        raise REQUESTS_TIMEOUT_ERROR
+
+    for error in errors:
+        if isinstance(error, int):
+            mock_requests.add(
+                method=responses.GET,
+                url=url,
+                status=error,
+            )
+        elif error == REQUESTS_CONNECTION_ERROR:
+            mock_requests.add_callback(
+                method=responses.GET,
+                url=url,
+                callback=callback_connection_error,
+            )
+        elif error == REQUESTS_TIMEOUT_ERROR:
+            mock_requests.add_callback(
+                method=responses.GET,
+                url=url,
+                callback=callback_timeout,
+            )
+        else:
+            assert False
+
+
+def regex_for_http_error(error):
+    if isinstance(error, int):
+        return f".*{error!s}.*"
+
+    # Returns 'Fake connection error' or 'Fake connection timeout'
+    return f".*{error!s}.*"
+
+
 class TestFetchMetadataWithApiFallback:
     default_url = (
         "http://169.254.169.254/metadata/instance?"
@@ -75,6 +137,9 @@ class TestFetchMetadataWithApiFallback:
     fallback_url = (
         "http://169.254.169.254/metadata/instance?api-version=2019-06-01"
     )
+
+    # Early versions of responses do not appreciate the parameters...
+    base_url = "http://169.254.169.254/metadata/instance"
     headers = {"Metadata": "true"}
     timeout = 30
 
@@ -82,14 +147,17 @@ class TestFetchMetadataWithApiFallback:
     def test_basic(
         self,
         caplog,
-        mock_requests_session_request,
         retry_deadline,
         wrapped_readurl,
+        mock_requests,
     ):
         fake_md = {"foo": {"bar": []}}
-        mock_requests_session_request.side_effect = [
-            mock.Mock(content=json.dumps(fake_md)),
-        ]
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            json=fake_md,
+            status=200,
+        )
 
         md = imds.fetch_metadata_with_api_fallback(
             retry_deadline=retry_deadline
@@ -121,17 +189,20 @@ class TestFetchMetadataWithApiFallback:
 
     @pytest.mark.parametrize("retry_deadline", [0.0, 1.0, 60.0])
     def test_basic_fallback(
-        self,
-        caplog,
-        mock_requests_session_request,
-        retry_deadline,
-        wrapped_readurl,
+        self, caplog, retry_deadline, wrapped_readurl, mock_requests
     ):
         fake_md = {"foo": {"bar": []}}
-        mock_requests_session_request.side_effect = [
-            UrlError("No IMDS version", code=400),
-            mock.Mock(content=json.dumps(fake_md)),
-        ]
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            status=400,
+        )
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            json=fake_md,
+            status=200,
+        )
 
         md = imds.fetch_metadata_with_api_fallback(
             retry_deadline=retry_deadline
@@ -166,7 +237,14 @@ class TestFetchMetadataWithApiFallback:
             (
                 LOG_PATH,
                 logging.WARNING,
-                "Failed to fetch metadata from IMDS: No IMDS version",
+                StringMatch("Polling IMDS failed attempt 1 with.*400.*"),
+            ),
+            (
+                LOG_PATH,
+                logging.WARNING,
+                StringMatch(
+                    "Failed to fetch metadata from IMDS:.*400 Client Error.*"
+                ),
             ),
             (
                 LOG_PATH,
@@ -181,44 +259,53 @@ class TestFetchMetadataWithApiFallback:
             (
                 "cloudinit.url_helper",
                 logging.DEBUG,
-                StringMatch("Read from.*"),
+                f"Read from {self.fallback_url} (200, 20b) after 1 attempts",
             ),
         ]
 
     @pytest.mark.parametrize(
         "error",
         [
-            fake_http_error_for_code(404),
-            fake_http_error_for_code(410),
-            fake_http_error_for_code(429),
-            fake_http_error_for_code(500),
-            requests.ConnectionError("Fake connection error"),
-            requests.Timeout("Fake connection timeout"),
+            404,
+            410,
+            429,
+            500,
+            REQUESTS_CONNECTION_ERROR,
+            REQUESTS_TIMEOUT_ERROR,
         ],
     )
-    @pytest.mark.parametrize("max_attempts,retry_deadline", [(2, 1.0)])
+    @pytest.mark.parametrize("error_count,retry_deadline", [(1, 2.0)])
     def test_will_retry_errors(
         self,
         caplog,
-        max_attempts,
         retry_deadline,
-        mock_requests_session_request,
+        mock_requests,
         mock_url_helper_time_sleep,
         error,
+        error_count,
     ):
         fake_md = {"foo": {"bar": []}}
-        mock_requests_session_request.side_effect = [
-            error,
-            mock.Mock(content=json.dumps(fake_md)),
-        ]
+        add_errors_to_mock_requests(
+            mock_requests, [error] * error_count, self.base_url
+        )
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            json=fake_md,
+            status=200,
+        )
 
         md = imds.fetch_metadata_with_api_fallback(
             retry_deadline=retry_deadline
         )
 
         assert md == fake_md
-        assert len(mock_requests_session_request.mock_calls) == max_attempts
-        assert mock_url_helper_time_sleep.mock_calls == [mock.call(1)]
+        assert (
+            mock_url_helper_time_sleep.mock_calls
+            == [mock.call(1)] * error_count
+        )
+
+        error_regex = regex_for_http_error(error)
         assert caplog.record_tuples == [
             (
                 "cloudinit.url_helper",
@@ -227,10 +314,10 @@ class TestFetchMetadataWithApiFallback:
             ),
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 StringMatch(
-                    "Polling IMDS failed attempt 1 with exception:"
-                    f".*{error!s}.*"
+                    "Polling IMDS failed attempt 1 with exception: "
+                    f"{error_regex}"
                 ),
             ),
             (
@@ -255,24 +342,32 @@ class TestFetchMetadataWithApiFallback:
         self,
         caplog,
         retry_deadline,
-        mock_requests_session_request,
+        mock_requests,
         mock_url_helper_time_sleep,
     ):
-        error = fake_http_error_for_code(400)
         fake_md = {"foo": {"bar": []}}
-        mock_requests_session_request.side_effect = [
-            error,
-            fake_http_error_for_code(429),
-            mock.Mock(content=json.dumps(fake_md)),
-        ]
-        max_attempts = len(mock_requests_session_request.side_effect)
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            status=400,
+        )
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            status=429,
+        )
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            json=fake_md,
+            status=200,
+        )
 
         md = imds.fetch_metadata_with_api_fallback(
             retry_deadline=retry_deadline
         )
 
         assert md == fake_md
-        assert len(mock_requests_session_request.mock_calls) == max_attempts
         assert mock_url_helper_time_sleep.mock_calls == [mock.call(1)]
         assert caplog.record_tuples == [
             (
@@ -282,16 +377,17 @@ class TestFetchMetadataWithApiFallback:
             ),
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 StringMatch(
-                    "Polling IMDS failed attempt 1 with exception:"
-                    f".*{error!s}.*"
+                    "Polling IMDS failed attempt 1 with exception:.*400.*"
                 ),
             ),
             (
                 LOG_PATH,
                 logging.WARNING,
-                "Failed to fetch metadata from IMDS: fake error",
+                StringMatch(
+                    "Failed to fetch metadata from IMDS:.*400 Client Error.*"
+                ),
             ),
             (
                 LOG_PATH,
@@ -305,10 +401,9 @@ class TestFetchMetadataWithApiFallback:
             ),
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 StringMatch(
-                    "Polling IMDS failed attempt 1 with exception:"
-                    f".*{error!s}.*"
+                    "Polling IMDS failed attempt 1 with exception:.*429.*"
                 ),
             ),
             (
@@ -331,43 +426,51 @@ class TestFetchMetadataWithApiFallback:
     @pytest.mark.parametrize(
         "error",
         [
-            fake_http_error_for_code(404),
-            fake_http_error_for_code(410),
-            fake_http_error_for_code(429),
-            fake_http_error_for_code(500),
-            requests.ConnectionError("Fake connection error"),
-            requests.Timeout("Fake connection timeout"),
+            404,
+            410,
+            429,
+            500,
+            REQUESTS_CONNECTION_ERROR,
+            REQUESTS_TIMEOUT_ERROR,
         ],
     )
     @pytest.mark.parametrize(
-        "max_attempts,retry_deadline", [(1, 0.0), (2, 1.0), (301, 300.0)]
+        "error_count,retry_deadline", [(1, 0.0), (2, 1.0), (301, 300.0)]
     )
+    @pytest.mark.parametrize("max_connection_errors", [None, 1, 11])
     def test_retry_until_failure(
         self,
         error,
-        max_attempts,
+        error_count,
+        max_connection_errors,
         retry_deadline,
         caplog,
-        mock_requests_session_request,
+        mock_requests,
         mock_url_helper_time_sleep,
     ):
-        mock_requests_session_request.side_effect = error
+        add_errors_to_mock_requests(
+            mock_requests, [error] * error_count, self.base_url
+        )
 
         with pytest.raises(UrlError) as exc_info:
             imds.fetch_metadata_with_api_fallback(
-                retry_deadline=retry_deadline
+                max_connection_errors=max_connection_errors,
+                retry_deadline=retry_deadline,
             )
 
-        assert exc_info.value.cause == error
+        error_regex = regex_for_http_error(error)
+        assert re.search(error_regex, str(exc_info.value.cause))
 
-        # Connection errors max out at 11 attempts.
         max_attempts = (
-            11
+            min(max_connection_errors, int(retry_deadline) + 1)
             if isinstance(error, requests.ConnectionError)
-            and max_attempts > 11
-            else max_attempts
+            and isinstance(max_connection_errors, int)
+            else error_count
         )
-        assert len(mock_requests_session_request.mock_calls) == (max_attempts)
+
+        if max_attempts < error_count:
+            mock_requests.assert_all_requests_are_fired = False
+
         assert mock_url_helper_time_sleep.mock_calls == [mock.call(1)] * (
             max_attempts - 1
         )
@@ -376,10 +479,10 @@ class TestFetchMetadataWithApiFallback:
         assert logs == [
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 StringMatch(
                     f"Polling IMDS failed attempt {i} with exception:"
-                    f".*{error!s}.*"
+                    f"{error_regex}"
                 ),
             )
             for i in range(1, max_attempts + 1)
@@ -387,15 +490,17 @@ class TestFetchMetadataWithApiFallback:
             (
                 LOG_PATH,
                 logging.WARNING,
-                f"Failed to fetch metadata from IMDS: {error!s}",
+                StringMatch(
+                    f"Failed to fetch metadata from IMDS: {error_regex}",
+                ),
             )
         ]
 
     @pytest.mark.parametrize(
         "error",
         [
-            fake_http_error_for_code(403),
-            fake_http_error_for_code(501),
+            403,
+            501,
         ],
     )
     @pytest.mark.parametrize("retry_deadline", [0.0, 1.0, 60.0])
@@ -404,22 +509,22 @@ class TestFetchMetadataWithApiFallback:
         error,
         retry_deadline,
         caplog,
-        mock_requests_session_request,
+        mock_requests,
         mock_url_helper_time_sleep,
     ):
-        fake_md = {"foo": {"bar": []}}
-        mock_requests_session_request.side_effect = [
-            error,
-            mock.Mock(content=json.dumps(fake_md)),
-        ]
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            status=error,
+        )
 
         with pytest.raises(UrlError) as exc_info:
             imds.fetch_metadata_with_api_fallback(
                 retry_deadline=retry_deadline
             )
 
-        assert exc_info.value.cause == error
-        assert len(mock_requests_session_request.mock_calls) == 1
+        error_regex = regex_for_http_error(error)
+        assert re.search(error_regex, str(exc_info.value.cause))
         assert mock_url_helper_time_sleep.mock_calls == []
 
         assert caplog.record_tuples == [
@@ -430,30 +535,36 @@ class TestFetchMetadataWithApiFallback:
             ),
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 StringMatch(
                     "Polling IMDS failed attempt 1 with exception:"
-                    f".*{error!s}.*"
+                    f".*{error_regex}"
                 ),
             ),
             (
                 LOG_PATH,
                 logging.WARNING,
-                f"Failed to fetch metadata from IMDS: {error!s}",
+                StringMatch(
+                    f"Failed to fetch metadata from IMDS: {error_regex!s}"
+                ),
             ),
         ]
 
+    @pytest.mark.parametrize("body", ["", "invalid", "<tag></tag>"])
     @pytest.mark.parametrize("retry_deadline", [0.0, 1.0, 60.0])
     def test_non_json_repsonse(
         self,
+        body,
         retry_deadline,
         caplog,
-        mock_requests_session_request,
+        mock_requests,
         wrapped_readurl,
     ):
-        mock_requests_session_request.side_effect = [
-            mock.Mock(content=b"bad data")
-        ]
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            body=body,
+        )
 
         with pytest.raises(ValueError):
             imds.fetch_metadata_with_api_fallback(
@@ -481,6 +592,47 @@ class TestFetchMetadataWithApiFallback:
             )
         ]
 
+    def test_logs_all_errors(
+        self,
+        caplog,
+        mock_requests,
+        mock_url_helper_time_sleep,
+    ):
+        fake_md = {"foo": {"bar": []}}
+
+        errors = (
+            [404] * 10
+            + [410] * 10
+            + [429] * 10
+            + [500] * 10
+            + [REQUESTS_CONNECTION_ERROR] * 10
+            + [REQUESTS_TIMEOUT_ERROR] * 10
+        )
+
+        add_errors_to_mock_requests(mock_requests, errors, self.base_url)
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            json=fake_md,
+        )
+
+        md = imds.fetch_metadata_with_api_fallback(retry_deadline=300)
+
+        assert md == fake_md
+        assert mock_url_helper_time_sleep.mock_calls == [mock.call(1)] * (
+            len(errors)
+        )
+
+        expected_logs = [
+            StringMatch(
+                f"Polling IMDS failed attempt {i} with exception:"
+                f"{regex_for_http_error(error)}"
+            )
+            for i, error in enumerate(errors, start=1)
+        ]
+        logs = [t[2] for t in caplog.record_tuples if "Polling IMDS" in t[2]]
+        assert logs == expected_logs
+
 
 class TestFetchReprovisionData:
     url = (
@@ -490,6 +642,9 @@ class TestFetchReprovisionData:
     headers = {"Metadata": "true"}
     timeout = 30
 
+    # Early versions of responses do not appreciate the parameters...
+    base_url = "http://169.254.169.254/metadata/reprovisiondata"
+
     def test_basic(
         self,
         caplog,
@@ -533,6 +688,7 @@ class TestFetchReprovisionData:
         [
             fake_http_error_for_code(404),
             fake_http_error_for_code(410),
+            fake_http_error_for_code(429),
         ],
     )
     @pytest.mark.parametrize("failures", [1, 5, 100, 1000])
@@ -566,7 +722,7 @@ class TestFetchReprovisionData:
         backoff_logs = [
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 f"Polling IMDS failed attempt {i} with exception: "
                 f"{wrapped_error!r}",
             )
@@ -591,6 +747,7 @@ class TestFetchReprovisionData:
         [
             fake_http_error_for_code(404),
             fake_http_error_for_code(410),
+            fake_http_error_for_code(429),
         ],
     )
     @pytest.mark.parametrize("failures", [1, 5, 100, 1000])
@@ -632,7 +789,7 @@ class TestFetchReprovisionData:
         backoff_logs = [
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 f"Polling IMDS failed attempt {i} with exception: "
                 f"{wrapped_error!r}",
             )
@@ -642,7 +799,7 @@ class TestFetchReprovisionData:
         assert caplog.record_tuples == backoff_logs + [
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 f"Polling IMDS failed attempt {failures+1} with exception: "
                 f"{exc_info.value!r}",
             ),
@@ -678,8 +835,50 @@ class TestFetchReprovisionData:
         assert caplog.record_tuples == [
             (
                 LOG_PATH,
-                logging.INFO,
+                logging.WARNING,
                 "Polling IMDS failed attempt 1 with exception: "
                 f"{exc_info.value!r}",
             ),
         ]
+
+    def test_logs_unique_errors(
+        self,
+        caplog,
+        mock_requests,
+        mock_url_helper_time_sleep,
+    ):
+        content = b"ovf content"
+
+        errors = (
+            [404] * 10
+            + [410] * 10
+            + [429] * 10
+            + [404] * 10
+            + [410] * 10
+            + [429] * 10
+        )
+
+        add_errors_to_mock_requests(mock_requests, errors, self.base_url)
+        mock_requests.add(
+            method=responses.GET,
+            url=self.base_url,
+            body=content,
+        )
+
+        ovf = imds.fetch_reprovision_data()
+
+        assert ovf == content
+        assert mock_url_helper_time_sleep.mock_calls == [mock.call(1)] * (
+            len(errors)
+        )
+
+        backoff_logs = [
+            StringMatch(
+                f"Polling IMDS failed attempt {i} with exception:"
+                f"{regex_for_http_error(error)}"
+            )
+            for i, error in enumerate(errors, start=1)
+            if i == 1 or math.log2(i).is_integer() or (i - 1) % 10 == 0
+        ]
+        logs = [t[2] for t in caplog.record_tuples if "Polling IMDS" in t[2]]
+        assert logs == backoff_logs
