@@ -10,8 +10,10 @@ import requests
 import responses
 
 from cloudinit import helpers
+from cloudinit.net import activators
 from cloudinit.sources import DataSourceEc2 as ec2
 from tests.unittests import helpers as test_helpers
+from tests.unittests.util import MockDistro
 
 DYNAMIC_METADATA = {
     "instance-identity": {
@@ -170,6 +172,28 @@ NIC2_MD = {
     "vpc-ipv4-cidr-blocks": "172.31.0.0/16",
 }
 
+NIC2_MD_IPV4_IPV6_MULTI_IP = {
+    "device-number": "1",
+    "interface-id": "eni-043cdce36ded5e79f",
+    "ipv6s": [
+        "2600:1f16:292:100:c187:593c:4349:136",
+        "2600:1f16:292:100:f153:12a3:c37c:11f9",
+    ],
+    "local-hostname": "ip-172-31-47-221.us-east-2.compute.internal",
+    "local-ipv4s": "172.31.47.221",
+    "mac": "0a:75:69:92:e2:16",
+    "owner-id": "329910648901",
+    "security-group-ids": "sg-0d68fef37d8cc9b77",
+    "security-groups": "launch-wizard-17",
+    "subnet-id": "subnet-9d7ba0d1",
+    "subnet-ipv4-cidr-block": "172.31.32.0/20",
+    "subnet-ipv6-cidr-blocks": "2600:1f16:292:100::/64",
+    "vpc-id": "vpc-a07f62c8",
+    "vpc-ipv4-cidr-block": "172.31.0.0/16",
+    "vpc-ipv4-cidr-blocks": "172.31.0.0/16",
+    "vpc-ipv6-cidr-blocks": "2600:1f16:292:100::/56",
+}
+
 SECONDARY_IP_METADATA_2018_09_24 = {
     "ami-id": "ami-0986c2ac728528ac2",
     "ami-launch-index": "0",
@@ -210,6 +234,7 @@ SECONDARY_IP_METADATA_2018_09_24 = {
     "services": {"domain": "amazonaws.com", "partition": "aws"},
 }
 
+M_PATH = "cloudinit.sources.DataSourceEc2."
 M_PATH_NET = "cloudinit.sources.DataSourceEc2.net."
 
 TAGS_METADATA_2021_03_23: dict = {
@@ -342,9 +367,11 @@ class TestEc2(test_helpers.ResponsesTestCase):
         p.start()
         self.addCleanup(p.stop)
 
-    def _setup_ds(self, sys_cfg, platform_data, md, md_version=None):
+    def _setup_ds(
+        self, sys_cfg, platform_data, md, md_version=None, distro=None
+    ):
         self.uris = []
-        distro = mock.MagicMock()
+        distro = distro or mock.MagicMock()
         distro.get_tmp_exec_path = self.tmp_dir
         paths = helpers.Paths({"run_dir": self.tmp})
         if sys_cfg is None:
@@ -479,6 +506,10 @@ class TestEc2(test_helpers.ResponsesTestCase):
         with mock.patch(patch_path) as m_get_interfaces_by_mac:
             with mock.patch(find_fallback_path) as m_find_fallback:
                 with mock.patch(get_interface_mac_path) as m_get_mac:
+                    dhcp_client = ds.distro.dhcp_client
+                    dhcp_client.dhcp_discovery.return_value = {
+                        "routers": "172.31.1.0"
+                    }
                     m_get_interfaces_by_mac.return_value = {mac1: "eth9"}
                     m_find_fallback.return_value = "eth9"
                     m_get_mac.return_value = mac1
@@ -577,7 +608,7 @@ class TestEc2(test_helpers.ResponsesTestCase):
         )
         mac1 = "06:17:04:d7:26:09"  # Defined in DEFAULT_METADATA
         get_interface_mac_path = M_PATH_NET + "get_interfaces_by_mac"
-        ds.fallback_nic = "eth9"
+        ds.distro.fallback_nic = "eth9"
         with mock.patch(get_interface_mac_path) as m_get_interfaces_by_mac:
             m_get_interfaces_by_mac.return_value = {mac1: "eth9"}
             nc = ds.network_config  # Will re-crawl network metadata
@@ -846,7 +877,7 @@ class TestEc2(test_helpers.ResponsesTestCase):
 
     @mock.patch("cloudinit.net.ephemeral.EphemeralIPv6Network")
     @mock.patch("cloudinit.net.ephemeral.EphemeralIPv4Network")
-    @mock.patch("cloudinit.net.find_fallback_nic")
+    @mock.patch("cloudinit.distros.net.find_fallback_nic")
     @mock.patch("cloudinit.net.ephemeral.maybe_perform_dhcp_discovery")
     @mock.patch("cloudinit.sources.DataSourceEc2.util.is_FreeBSD")
     def test_ec2_local_performs_dhcp_on_non_bsd(
@@ -861,20 +892,19 @@ class TestEc2(test_helpers.ResponsesTestCase):
 
         m_fallback_nic.return_value = "eth9"
         m_is_bsd.return_value = False
-        m_dhcp.return_value = [
-            {
-                "interface": "eth9",
-                "fixed-address": "192.168.2.9",
-                "routers": "192.168.2.1",
-                "subnet-mask": "255.255.255.0",
-                "broadcast-address": "192.168.2.255",
-            }
-        ]
+        m_dhcp.return_value = {
+            "interface": "eth9",
+            "fixed-address": "192.168.2.9",
+            "routers": "192.168.2.1",
+            "subnet-mask": "255.255.255.0",
+            "broadcast-address": "192.168.2.255",
+        }
         self.datasource = ec2.DataSourceEc2Local
         ds = self._setup_ds(
             platform_data=self.valid_platform_data,
             sys_cfg={"datasource": {"Ec2": {"strict_id": False}}},
             md={"md": DEFAULT_METADATA},
+            distro=MockDistro("", {}, {}),
         )
 
         ret = ds.get_data()
@@ -906,7 +936,6 @@ class TestEc2(test_helpers.ResponsesTestCase):
 
 
 class TestGetSecondaryAddresses(test_helpers.CiTestCase):
-
     mac = "06:17:04:d7:26:ff"
     with_logs = True
 
@@ -951,6 +980,144 @@ class TestGetSecondaryAddresses(test_helpers.CiTestCase):
             self.assertIn(log, logs)
 
 
+class TestBuildNicOrder:
+    @pytest.mark.parametrize(
+        ["macs_metadata", "macs", "expected"],
+        [
+            pytest.param({}, [], {}, id="all_empty"),
+            pytest.param(
+                {}, ["0a:f7:8d:96:f2:a1"], {}, id="empty_macs_metadata"
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "device-number": "0",
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    }
+                },
+                [],
+                {},
+                id="empty_macs",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                ["0a:f7:8d:96:f2:a1", "0a:0d:dd:44:cd:7b"],
+                {"0a:f7:8d:96:f2:a1": 0, "0a:0d:dd:44:cd:7b": 1},
+                id="no-device-number-info",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                ["0a:f7:8d:96:f2:a1"],
+                {"0a:f7:8d:96:f2:a1": 0},
+                id="no-device-number-info-subset",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "device-number": "0",
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "device-number": "1",
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                ["0a:f7:8d:96:f2:a1", "0a:0d:dd:44:cd:7b"],
+                {"0a:0d:dd:44:cd:7b": 0, "0a:f7:8d:96:f2:a1": 1},
+                id="device-numbers",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "network-card": "0",
+                        "device-number": "0",
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "network-card": "1",
+                        "device-number": "1",
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                    "0a:f7:8d:96:f2:a2": {
+                        "network-card": "2",
+                        "device-number": "1",
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                [
+                    "0a:f7:8d:96:f2:a1",
+                    "0a:0d:dd:44:cd:7b",
+                    "0a:f7:8d:96:f2:a2",
+                ],
+                {
+                    "0a:0d:dd:44:cd:7b": 0,
+                    "0a:f7:8d:96:f2:a1": 1,
+                    "0a:f7:8d:96:f2:a2": 2,
+                },
+                id="network-cardes",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "network-card": "0",
+                        "device-number": "0",
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "network-card": "1",
+                        "device-number": "1",
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                    "0a:f7:8d:96:f2:a2": {
+                        "device-number": "1",
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                [
+                    "0a:f7:8d:96:f2:a1",
+                    "0a:0d:dd:44:cd:7b",
+                    "0a:f7:8d:96:f2:a2",
+                ],
+                {
+                    "0a:0d:dd:44:cd:7b": 0,
+                    "0a:f7:8d:96:f2:a1": 1,
+                    "0a:f7:8d:96:f2:a2": 2,
+                },
+                id="network-card-partially-missing",
+            ),
+            pytest.param(
+                {
+                    "0a:0d:dd:44:cd:7b": {
+                        "mac": "0a:0d:dd:44:cd:7b",
+                    },
+                    "0a:f7:8d:96:f2:a1": {
+                        "mac": "0a:f7:8d:96:f2:a1",
+                    },
+                },
+                ["0a:f7:8d:96:f2:a9"],
+                {},
+                id="macs-not-in-md",
+            ),
+        ],
+    )
+    def test_build_nic_order(self, macs_metadata, macs, expected):
+        assert expected == ec2._build_nic_order(macs_metadata, macs)
+
+
 class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
     def setUp(self):
         super(TestConvertEc2MetadataNetworkConfig, self).setUp()
@@ -980,10 +1147,11 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                self.network_metadata, macs_to_nics
+                self.network_metadata, distro, macs_to_nics
             ),
         )
 
@@ -1005,10 +1173,11 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_ipv6, macs_to_nics
+                network_metadata_ipv6, distro, macs_to_nics
             ),
         )
 
@@ -1030,10 +1199,11 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_ipv6, macs_to_nics
+                network_metadata_ipv6, distro, macs_to_nics
             ),
         )
 
@@ -1056,10 +1226,11 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_ipv6, macs_to_nics, fallback_nic="eth9"
+                network_metadata_ipv6, distro, macs_to_nics
             ),
         )
 
@@ -1082,15 +1253,18 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_both, macs_to_nics
+                network_metadata_both, distro, macs_to_nics
             ),
         )
 
-    def test_convert_ec2_metadata_network_config_handles_multiple_nics(self):
-        """DHCP route-metric increases on secondary NICs for IPv4 and IPv6."""
+    def test_convert_ec2_metadata_network_config_multi_nics_ipv4(self):
+        """DHCP route-metric increases on secondary NICs for IPv4 and IPv6.
+        Source-routing configured for secondary NICs (routing-policy and extra
+        routing table)."""
         mac2 = "06:17:04:d7:26:08"
         macs_to_nics = {self.mac1: "eth9", mac2: "eth10"}
         network_metadata_both = copy.deepcopy(self.network_metadata)
@@ -1115,15 +1289,110 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                     "match": {"macaddress": mac2},
                     "set-name": "eth10",
                     "dhcp4": True,
-                    "dhcp4-overrides": {"route-metric": 200},
+                    "dhcp4-overrides": {
+                        "route-metric": 200,
+                        "use-routes": True,
+                    },
                     "dhcp6": False,
+                    "routes": [
+                        # via DHCP gateway
+                        {"to": "0.0.0.0/0", "via": "172.31.1.0", "table": 101},
+                        # to NIC2_MD["subnet-ipv4-cidr-block"]
+                        {"to": "172.31.32.0/20", "table": 101},
+                    ],
+                    "routing-policy": [
+                        # NIC2_MD["local-ipv4s"]
+                        {"from": "172.31.47.221", "table": 101}
+                    ],
                 },
             },
         }
+        distro = mock.Mock()
+        distro.network_activator = activators.NetplanActivator
+        distro.dhcp_client.dhcp_discovery.return_value = {
+            "routers": "172.31.1.0"
+        }
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_both, macs_to_nics
+                network_metadata_both, distro, macs_to_nics
+            ),
+        )
+
+    def test_convert_ec2_metadata_network_config_multi_nics_ipv4_ipv6_multi_ip(
+        self,
+    ):
+        """DHCP route-metric increases on secondary NICs for IPv4 and IPv6.
+        Source-routing configured for secondary NICs (routing-policy and extra
+        routing table)."""
+        mac2 = "06:17:04:d7:26:08"
+        macs_to_nics = {self.mac1: "eth9", mac2: "eth10"}
+        network_metadata_both = copy.deepcopy(self.network_metadata)
+        # Add 2nd nic info
+        network_metadata_both["interfaces"]["macs"][
+            mac2
+        ] = NIC2_MD_IPV4_IPV6_MULTI_IP
+        nic1_metadata = network_metadata_both["interfaces"]["macs"][self.mac1]
+        nic1_metadata["ipv6s"] = "2620:0:1009:fd00:e442:c88d:c04d:dc85/64"
+        nic1_metadata.pop("public-ipv4s")  # No public-ipv4 IPs in cfg
+        nic1_metadata["local-ipv4s"] = "10.0.0.42"  # Local ipv4 only on vpc
+        expected = {
+            "version": 2,
+            "ethernets": {
+                "eth9": {
+                    "dhcp4": True,
+                    "dhcp4-overrides": {"route-metric": 100},
+                    "dhcp6": True,
+                    "match": {"macaddress": "06:17:04:d7:26:09"},
+                    "set-name": "eth9",
+                    "dhcp6-overrides": {"route-metric": 100},
+                },
+                "eth10": {
+                    "dhcp4": True,
+                    "dhcp4-overrides": {
+                        "route-metric": 200,
+                        "use-routes": True,
+                    },
+                    "dhcp6": True,
+                    "match": {"macaddress": "06:17:04:d7:26:08"},
+                    "set-name": "eth10",
+                    "routes": [
+                        # via DHCP gateway
+                        {"to": "0.0.0.0/0", "via": "172.31.1.0", "table": 101},
+                        # to NIC2_MD["subnet-ipv4-cidr-block"]
+                        {"to": "172.31.32.0/20", "table": 101},
+                        # to NIC2_MD["subnet-ipv6-cidr-blocks"]
+                        {"to": "2600:1f16:292:100::/64", "table": 101},
+                    ],
+                    "routing-policy": [
+                        # NIC2_MD["local-ipv4s"]
+                        {"from": "172.31.47.221", "table": 101},
+                        {
+                            "from": "2600:1f16:292:100:c187:593c:4349:136",
+                            "table": 101,
+                        },
+                        {
+                            "from": "2600:1f16:292:100:f153:12a3:c37c:11f9",
+                            "table": 101,
+                        },
+                    ],
+                    "dhcp6-overrides": {
+                        "route-metric": 200,
+                        "use-routes": True,
+                    },
+                    "addresses": ["2600:1f16:292:100:f153:12a3:c37c:11f9/128"],
+                },
+            },
+        }
+        distro = mock.Mock()
+        distro.network_activator = activators.NetplanActivator
+        distro.dhcp_client.dhcp_discovery.return_value = {
+            "routers": "172.31.1.0"
+        }
+        self.assertEqual(
+            expected,
+            ec2.convert_ec2_metadata_network_config(
+                network_metadata_both, distro, macs_to_nics
             ),
         )
 
@@ -1144,10 +1413,11 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
                 }
             },
         }
+        distro = mock.Mock()
         self.assertEqual(
             expected,
             ec2.convert_ec2_metadata_network_config(
-                network_metadata_both, macs_to_nics
+                network_metadata_both, distro, macs_to_nics
             ),
         )
 
@@ -1165,11 +1435,14 @@ class TestConvertEc2MetadataNetworkConfig(test_helpers.CiTestCase):
             },
         }
         patch_path = M_PATH_NET + "get_interfaces_by_mac"
+        distro = mock.Mock()
         with mock.patch(patch_path) as m_get_interfaces_by_mac:
             m_get_interfaces_by_mac.return_value = {self.mac1: "eth9"}
             self.assertEqual(
                 expected,
-                ec2.convert_ec2_metadata_network_config(self.network_metadata),
+                ec2.convert_ec2_metadata_network_config(
+                    self.network_metadata, distro
+                ),
             )
 
 
