@@ -67,7 +67,7 @@ class TestOpenNebulaDataSource(CiTestCase):
 
     def tearDown(self):
         ds.switch_user_cmd = self.switch_user_cmd_real
-        super(TestOpenNebulaDataSource, self).tearDown()
+        super().tearDown()
 
     def test_get_data_non_contextdisk(self):
         orig_find_devs_with = util.find_devs_with
@@ -1066,6 +1066,3 @@ def populate_context_dir(path, variables):
     for k, v in variables.items():
         data += "%s='%s'\n" % (k.upper(), v.replace(r"'", r"'\''"))
     populate_dir(path, {"context.sh": data})
-
-
-# vi: ts=4 expandtab
