@@ -16,6 +16,7 @@ import pytest
 from cloudinit import dmi, helpers, safeyaml, settings, util
 from cloudinit.sources import DataSourceVMware
 from cloudinit.sources.helpers.vmware.imc import guestcust_util
+from cloudinit.subp import ProcessExecutionError
 from tests.unittests.helpers import (
     CiTestCase,
     FilesystemMockingTestCase,
@@ -100,7 +101,6 @@ class TestDataSourceVMware(CiTestCase):
 
     def test_no_data_access_method(self):
         ds = get_ds(self.tmp)
-        ds.vmware_rpctool = None
         with mock.patch(
             "cloudinit.sources.DataSourceVMware.is_vmware_platform",
             return_value=False,
@@ -238,7 +238,7 @@ class TestDataSourceVMwareEnvVars(FilesystemMockingTestCase):
 
     def tearDown(self):
         del os.environ[DataSourceVMware.VMX_GUESTINFO]
-        return super(TestDataSourceVMwareEnvVars, self).tearDown()
+        return super().tearDown()
 
     def create_system_files(self):
         rootd = self.tmp_dir()
@@ -252,7 +252,6 @@ class TestDataSourceVMwareEnvVars(FilesystemMockingTestCase):
 
     def assert_get_data_ok(self, m_fn, m_fn_call_count=6):
         ds = get_ds(self.tmp)
-        ds.vmware_rpctool = None
         ret = ds.get_data()
         self.assertTrue(ret)
         self.assertEqual(m_fn_call_count, m_fn.call_count)
@@ -381,7 +380,6 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
 
     def assert_get_data_ok(self, m_fn, m_fn_call_count=6):
         ds = get_ds(self.tmp)
-        ds.vmware_rpctool = "vmware-rpctool"
         ret = ds.get_data()
         self.assertTrue(ret)
         self.assertEqual(m_fn_call_count, m_fn.call_count)
@@ -400,7 +398,9 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
         self.assertEqual(system_type, PRODUCT_NAME)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_subplatform(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_subplatform(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         m_fn.side_effect = [VMW_METADATA_YAML, "", "", "", "", ""]
         ds = self.assert_get_data_ok(m_fn, m_fn_call_count=4)
         self.assertEqual(
@@ -413,17 +413,53 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
         )
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_userdata_only(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_with_vmware_rpctool(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
+        m_fn.side_effect = [VMW_METADATA_YAML, "", "", ""]
+        self.assert_get_data_ok(m_fn, m_fn_call_count=4)
+
+    @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
+    @mock.patch("cloudinit.sources.DataSourceVMware.exec_vmware_rpctool")
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_non_zero_exit_code_fallback_to_vmtoolsd(
+        self, m_which_fn, m_exec_vmware_rpctool_fn, m_fn
+    ):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
+        m_exec_vmware_rpctool_fn.side_effect = ProcessExecutionError(
+            exit_code=1
+        )
+        m_fn.side_effect = [VMW_METADATA_YAML, "", "", ""]
+        self.assert_get_data_ok(m_fn, m_fn_call_count=4)
+
+    @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
+    @mock.patch("cloudinit.sources.DataSourceVMware.exec_vmware_rpctool")
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_vmware_rpctool_not_found_fallback_to_vmtoolsd(
+        self, m_which_fn, m_exec_vmware_rpctool_fn, m_fn
+    ):
+        m_which_fn.side_effect = ["vmtoolsd", None]
+        m_fn.side_effect = [VMW_METADATA_YAML, "", "", ""]
+        self.assert_get_data_ok(m_fn, m_fn_call_count=4)
+
+    @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_userdata_only(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         m_fn.side_effect = ["", VMW_USERDATA_YAML, "", ""]
         self.assert_get_data_ok(m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_vendordata_only(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_vendordata_only(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         m_fn.side_effect = ["", "", VMW_VENDORDATA_YAML, ""]
         self.assert_get_data_ok(m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_metadata_single_ssh_key(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_metadata_single_ssh_key(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         metadata = DataSourceVMware.load_json_or_yaml(VMW_METADATA_YAML)
         metadata["public_keys"] = VMW_SINGLE_KEY
         metadata_yaml = safeyaml.dumps(metadata)
@@ -431,7 +467,9 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
         self.assert_metadata(metadata, m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_metadata_multiple_ssh_keys(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_metadata_multiple_ssh_keys(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         metadata = DataSourceVMware.load_json_or_yaml(VMW_METADATA_YAML)
         metadata["public_keys"] = VMW_MULTIPLE_KEYS
         metadata_yaml = safeyaml.dumps(metadata)
@@ -439,19 +477,25 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
         self.assert_metadata(metadata, m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_metadata_base64(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_base64(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         data = base64.b64encode(VMW_METADATA_YAML.encode("utf-8"))
         m_fn.side_effect = [data, "base64", "", ""]
         self.assert_get_data_ok(m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_metadata_b64(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_b64(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         data = base64.b64encode(VMW_METADATA_YAML.encode("utf-8"))
         m_fn.side_effect = [data, "b64", "", ""]
         self.assert_get_data_ok(m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_metadata_gzip_base64(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_gzip_base64(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         data = VMW_METADATA_YAML.encode("utf-8")
         data = gzip.compress(data)
         data = base64.b64encode(data)
@@ -459,7 +503,9 @@ class TestDataSourceVMwareGuestInfo(FilesystemMockingTestCase):
         self.assert_get_data_ok(m_fn, m_fn_call_count=4)
 
     @mock.patch("cloudinit.sources.DataSourceVMware.guestinfo_get_value")
-    def test_get_data_metadata_gz_b64(self, m_fn):
+    @mock.patch("cloudinit.sources.DataSourceVMware.which")
+    def test_get_data_metadata_gz_b64(self, m_which_fn, m_fn):
+        m_which_fn.side_effect = ["vmtoolsd", "vmware-rpctool"]
         data = VMW_METADATA_YAML.encode("utf-8")
         data = gzip.compress(data)
         data = base64.b64encode(data)
@@ -494,7 +540,6 @@ class TestDataSourceVMwareGuestInfo_InvalidPlatform(FilesystemMockingTestCase):
 
         m_fn.side_effect = [VMW_METADATA_YAML, "", "", "", "", ""]
         ds = get_ds(self.tmp)
-        ds.vmware_rpctool = "vmware-rpctool"
         ret = ds.get_data()
         self.assertFalse(ret)
 
@@ -1217,8 +1262,4 @@ def get_ds(temp_dir):
     ds = DataSourceVMware.DataSourceVMware(
         settings.CFG_BUILTIN, None, helpers.Paths({"run_dir": temp_dir})
     )
-    ds.vmware_rpctool = "vmware-rpctool"
     return ds
-
-
-# vi: ts=4 expandtab
