@@ -25,7 +25,8 @@ from cloudinit.sources.helpers.vmware.imc.guestcust_util import (
     get_network_data_from_vmware_cust_cfg,
     get_non_network_data_from_vmware_cust_cfg,
 )
-from tests.unittests.helpers import CiTestCase, cloud_init_project_dir
+from tests.helpers import cloud_init_project_dir
+from tests.unittests.helpers import CiTestCase
 
 logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
 logger = logging.getLogger(__name__)
@@ -625,6 +626,3 @@ class TestVmwareNetConfig(CiTestCase):
         cf._insertKey("CLOUDINIT|USERDATA", "test-userdata")
         conf = Config(cf)
         self.assertEqual("test-userdata", conf.user_data_name)
-
-
-# vi: ts=4 expandtab
