@@ -293,7 +293,7 @@ class TestCmdlineUrl:
         lvl, msg = main.attempt_cmdline_url(
             fpath, network=True, cmdline=cmdline
         )
-        assert util.load_file(fpath, decode=False) == payload
+        assert util.load_binary_file(fpath) == payload
         assert logging.INFO == lvl
         assert url in msg
 
@@ -310,7 +310,7 @@ class TestCmdlineUrl:
         lvl, msg = main.attempt_cmdline_url(
             fpath, network=True, cmdline=cmdline
         )
-        assert util.load_file(fpath, decode=False) == payload
+        assert util.load_binary_file(fpath) == payload
         assert logging.INFO == lvl
         assert url in msg
 
@@ -341,6 +341,3 @@ class TestCmdlineUrl:
         assert logging.WARN == lvl
         assert url in msg
         assert False is os.path.exists(fpath)
-
-
-# vi: ts=4 expandtab
