@@ -2,6 +2,7 @@
 
 import contextlib
 import io
+import logging
 import os
 from collections import namedtuple
 
@@ -9,7 +10,6 @@ import pytest
 
 from cloudinit import helpers
 from cloudinit.cmd import main as cli
-from cloudinit.util import load_file, load_json
 from tests.unittests import helpers as test_helpers
 
 mock = test_helpers.mock
@@ -62,14 +62,18 @@ class TestCLI:
             cli.status_wrapper(name, myargs, data_d, link_d)
         assert [] == my_action.call_args_list
 
-    def test_status_wrapper_init_local_writes_fresh_status_info(self, tmpdir):
+    @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
+    def test_status_wrapper_init_local_writes_fresh_status_info(
+        self,
+        m_json,
+        tmpdir,
+    ):
         """When running in init-local mode, status_wrapper writes status.json.
 
         Old status and results artifacts are also removed.
         """
         data_d = tmpdir.join("data")
         link_d = tmpdir.join("link")
-        status_link = link_d.join("status.json")
         # Write old artifacts which will be removed or updated.
         for _dir in data_d, link_d:
             test_helpers.populate_dir(
@@ -85,7 +89,15 @@ class TestCLI:
         myargs = FakeArgs(("ignored_name", myaction), True, "bogusmode")
         cli.status_wrapper("init", myargs, data_d, link_d)
         # No errors reported in status
-        status_v1 = load_json(load_file(status_link))["v1"]
+        status_v1 = m_json.call_args_list[1][0][1]["v1"]
+        assert status_v1.keys() == {
+            "datasource",
+            "init-local",
+            "init",
+            "modules-config",
+            "modules-final",
+            "stage",
+        }
         assert ["an error"] == status_v1["init-local"]["errors"]
         assert "SomeDatasource" == status_v1["datasource"]
         assert False is os.path.exists(
@@ -95,7 +107,10 @@ class TestCLI:
             link_d.join("result.json")
         ), "unexpected result.json link found"
 
-    def test_status_wrapper_init_local_honor_cloud_dir(self, mocker, tmpdir):
+    @mock.patch("cloudinit.cmd.main.atomic_helper.write_json")
+    def test_status_wrapper_init_local_honor_cloud_dir(
+        self, m_json, mocker, tmpdir
+    ):
         """When running in init-local mode, status_wrapper honors cloud_dir."""
         cloud_dir = tmpdir.join("cloud")
         paths = helpers.Paths({"cloud_dir": str(cloud_dir)})
@@ -111,8 +126,9 @@ class TestCLI:
 
         myargs = FakeArgs(("ignored_name", myaction), True, "bogusmode")
         cli.status_wrapper("init", myargs, link_d=link_d)  # No explicit data_d
+
         # Access cloud_dir directly
-        status_v1 = load_json(load_file(data_d.join("status.json")))["v1"]
+        status_v1 = m_json.call_args_list[1][0][1]["v1"]
         assert ["an_error"] == status_v1["init-local"]["errors"]
         assert "SomeDatasource" == status_v1["datasource"]
         assert False is os.path.exists(
@@ -156,6 +172,45 @@ class TestCLI:
         for subcommand in expected_subcommands:
             assert subcommand in err
 
+    @pytest.mark.parametrize(
+        "subcommand,log_to_stderr,mocks",
+        (
+            ("init", False, [mock.patch("cloudinit.cmd.main.status_wrapper")]),
+            (
+                "modules",
+                False,
+                [mock.patch("cloudinit.cmd.main.status_wrapper")],
+            ),
+            (
+                "schema",
+                True,
+                [
+                    mock.patch(
+                        "cloudinit.stages.Init._read_cfg", return_value={}
+                    ),
+                    mock.patch("cloudinit.config.schema.handle_schema_args"),
+                ],
+            ),
+        ),
+    )
+    @mock.patch("cloudinit.cmd.main.setup_basic_logging")
+    def test_subcommands_log_to_stderr_via_setup_basic_logging(
+        self, setup_basic_logging, subcommand, log_to_stderr, mocks
+    ):
+        """setup_basic_logging is called for modules to use stderr
+
+        Subcommands with exception of 'init'  and 'modules' use
+        setup_basic_logging to direct logged errors to stderr.
+        """
+        with contextlib.ExitStack() as mockstack:
+            for mymock in mocks:
+                mockstack.enter_context(mymock)
+            self._call_main(["cloud-init", subcommand])
+        if log_to_stderr:
+            setup_basic_logging.assert_called_once_with(logging.WARNING)
+        else:
+            setup_basic_logging.assert_not_called()
+
     @pytest.mark.parametrize("subcommand", ["init", "modules"])
     @mock.patch("cloudinit.cmd.main.status_wrapper")
     def test_modules_subcommand_parser(self, m_status_wrapper, subcommand):
@@ -235,7 +290,7 @@ class TestCLI:
         assert (
             "Error:\n"
             "Expected one of --config-file, --system or --docs arguments\n"
-            == err
+            in err
         )
 
     @pytest.mark.parametrize(
