@@ -1,9 +1,12 @@
 # This file is part of cloud-init. See LICENSE file for license information.
+# pylint: disable=attribute-defined-outside-init
 
 import logging
+import re
 from functools import partial
 from threading import Event
 from time import process_time
+from unittest.mock import ANY, call
 
 import pytest
 import requests
@@ -11,14 +14,12 @@ import responses
 
 from cloudinit import util, version
 from cloudinit.url_helper import (
-    NOT_FOUND,
     REDACTED,
     UrlError,
     UrlResponse,
     dual_stack,
     oauth_headers,
     read_file_or_url,
-    retry_on_url_exc,
     wait_for_url,
 )
 from tests.unittests.helpers import CiTestCase, mock, skipIf
@@ -257,25 +258,6 @@ class TestReadFileOrUrlParameters:
         assert response._response == m_response
 
 
-class TestRetryOnUrlExc(CiTestCase):
-    def test_do_not_retry_non_urlerror(self):
-        """When exception is not UrlError return False."""
-        myerror = IOError("something unexcpected")
-        self.assertFalse(retry_on_url_exc(msg="", exc=myerror))
-
-    def test_perform_retries_on_not_found(self):
-        """When exception is UrlError with a 404 status code return True."""
-        myerror = UrlError(
-            cause=RuntimeError("something was not found"), code=NOT_FOUND
-        )
-        self.assertTrue(retry_on_url_exc(msg="", exc=myerror))
-
-    def test_perform_retries_on_timeout(self):
-        """When exception is a requests.Timout return True."""
-        myerror = UrlError(cause=requests.Timeout("something timed out"))
-        self.assertTrue(retry_on_url_exc(msg="", exc=myerror))
-
-
 def assert_time(func, max_time=1):
     """Assert function time is bounded by a max (default=1s)
 
@@ -465,20 +447,72 @@ class TestDualStack:
         """Assert expected call intervals occur"""
         stagger = 0.1
         with mock.patch(M_PATH + "_run_func_with_delay") as delay_func:
+
+            def identity_of_first_arg(x, _):
+                return x
+
             dual_stack(
-                lambda x, _y: x,
+                identity_of_first_arg,
                 ["you", "and", "me", "and", "dog"],
                 stagger_delay=stagger,
                 timeout=1,
             )
 
-            # ensure that stagger delay for each subsequent call is:
+            # ensure that stagger delay for each call is made with args:
             # [ 0 * N, 1 * N, 2 * N, 3 * N, 4 * N, 5 * N] where N = stagger
             # it appears that without an explicit wait/join we can't assert
             # number of calls
-            for delay, call_item in enumerate(delay_func.call_args_list):
-                _, kwargs = call_item
-                assert stagger * delay == kwargs.get("delay")
+            calls = [
+                call(
+                    func=identity_of_first_arg,
+                    addr="you",
+                    timeout=1,
+                    event=ANY,
+                    delay=stagger * 0,
+                ),
+                call(
+                    func=identity_of_first_arg,
+                    addr="and",
+                    timeout=1,
+                    event=ANY,
+                    delay=stagger * 1,
+                ),
+                call(
+                    func=identity_of_first_arg,
+                    addr="me",
+                    timeout=1,
+                    event=ANY,
+                    delay=stagger * 2,
+                ),
+                call(
+                    func=identity_of_first_arg,
+                    addr="and",
+                    timeout=1,
+                    event=ANY,
+                    delay=stagger * 3,
+                ),
+                call(
+                    func=identity_of_first_arg,
+                    addr="dog",
+                    timeout=1,
+                    event=ANY,
+                    delay=stagger * 4,
+                ),
+            ]
+            num_calls = 0
+            for call_instance in calls:
+                if call_instance in delay_func.call_args_list:
+                    num_calls += 1
+
+            # we can't know the order of the submitted functions' execution
+            # we can't know how many of the submitted functions get called
+            # in advance
+            #
+            # we _do_ know what the possible arg combinations are
+            # we _do_ know from the mocked function how many got called
+            # assert that all calls that occurred had known valid arguments
+            # by checking for the correct number of matches
+            assert num_calls == len(delay_func.call_args_list)
 
 
 ADDR1 = "https://addr1/"
@@ -491,6 +525,21 @@ class TestUrlHelper:
     fail = "FAIL"
     event = Event()
 
+    @pytest.fixture
+    def retry_mocks(self, mocker):
+        self.mock_time_value = 0
+        m_readurl = mocker.patch(
+            f"{M_PATH}readurl", side_effect=self.readurl_side_effect
+        )
+        m_sleep = mocker.patch(
+            f"{M_PATH}time.sleep", side_effect=self.sleep_side_effect
+        )
+        mocker.patch(f"{M_PATH}time.time", side_effect=self.time_side_effect)
+
+        yield m_readurl, m_sleep
+
+        self.mock_time_value = 0
+
     @classmethod
     def response_wait(cls, _request):
         cls.event.wait(0.1)
@@ -551,7 +600,7 @@ class TestUrlHelper:
         assert response.encode() == response_contents
 
     @responses.activate
-    def test_timeout(self):
+    def test_timeout(self, caplog):
         """If no endpoint responds in time, expect no response"""
 
         self.event.clear()
@@ -561,7 +610,7 @@ class TestUrlHelper:
                 responses.GET,
                 address,
                 callback=(
-                    self.response_wait
+                    requests.ConnectTimeout
                     if "sleep" in address
                     else self.response_nowait
                 ),
@@ -579,6 +628,74 @@ class TestUrlHelper:
         self.event.set()
         assert not url
         assert not response_contents
+        assert re.search(
+            r"open 'https:\/\/sleep1\/'.*Timed out", caplog.text, re.DOTALL
+        )
 
+    def test_explicit_arguments(self, retry_mocks):
+        """Ensure that explicit arguments are respected"""
+        m_readurl, m_sleep = retry_mocks
+        wait_for_url(
+            urls=["http://localhost/"],
+            max_wait=23,
+            timeout=5,
+            sleep_time=3,
+        )
 
-# vi: ts=4 expandtab
+        assert len(m_readurl.call_args_list) == 3
+        assert len(m_sleep.call_args_list) == 2
+
+        for readurl_call in m_readurl.call_args_list:
+            assert readurl_call[1]["timeout"] == 5
+        for sleep_call in m_sleep.call_args_list:
+            assert sleep_call[0][0] == 3
+
+        # Call 1 starts 0
+        # Call 2 starts at 8-ish after 5 second timeout and 3 second sleep
+        # Call 3 starts at 16-ish for same reasons
+        # The 5 second timeout puts us at 21-ish and now we break
+        # because 21-ish + the sleep time puts us over max wait of 23
+        assert pytest.approx(self.mock_time_value) == 21
+
+    def test_shortened_timeout(self, retry_mocks):
+        """Test that we shorten the last timeout to align with max_wait"""
+        m_readurl, _m_sleep = retry_mocks
+        wait_for_url(
+            urls=["http://localhost/"], max_wait=10, timeout=9, sleep_time=0
+        )
+
+        assert len(m_readurl.call_args_list) == 2
+        assert m_readurl.call_args_list[-1][1]["timeout"] == pytest.approx(1)
+
+    def test_default_sleep_time(self, retry_mocks):
+        """Test default sleep behavior when not specified"""
+        _m_readurl, m_sleep = retry_mocks
+        wait_for_url(
+            urls=["http://localhost/"],
+            max_wait=50,
+            timeout=1,
+        )
+
+        expected_sleep_times = [1] * 5 + [2] * 5 + [3] * 5
+        actual_sleep_times = [
+            m_sleep.call_args_list[i][0][0]
+            for i in range(len(m_sleep.call_args_list))
+        ]
+        assert actual_sleep_times == expected_sleep_times
+
+    # These side effect methods are a way of having a somewhat predictable
+    # output for time.time(). Otherwise, we have to track too many calls
+    # to time.time() and unrelated changes to code being called could cause
+    # these tests to fail.
+    # 0.0000001 is added to simulate additional execution time but keep it
+    # small enough for pytest.approx() to work
+    def sleep_side_effect(self, sleep_time):
+        self.mock_time_value += sleep_time + 0.0000001
+
+    def time_side_effect(self):
+        return self.mock_time_value
+
+    def readurl_side_effect(self, *args, **kwargs):
+        if "timeout" in kwargs:
+            self.mock_time_value += kwargs["timeout"] + 0.0000001
+        raise UrlError("test")
